# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk

class Avatar( object ):
    '''this class represent an avatar'''
    
    def __init__( self, path, cache, mail, height = 96, width = 96, thumbHeight = 48, thumbWidth = 48 ):
        self.path = path
        self.cache = cache
        self.mail = mail.split( '@' )[ 0 ]
        self.height = height
        self.width = width
        self.thumbHeight = thumbHeight
        self.thumbWidth = thumbWidth
        
        pixbuf = gtk.gdk.pixbuf_new_from_file( path ) 
 
        # need to check so it doesn't get scaled again and again
        # if scale() wasn't a hack, it needn't be checked
        cWidth, cHeight = pixbuf.get_width(), pixbuf.get_height()
        if (cWidth == width and cHeight == height): 
            image = pixbuf 
        else: 
            image = self.scale( pixbuf, width, height ) 
         
        thumb = self.scale( pixbuf, thumbWidth, thumbHeight )

        
        self.imagePath = self.cache + os.sep + self.mail + '.png'
        self.thumbPath = self.cache + os.sep + self.mail + '_thumb.png'
        
        image.save( self.imagePath, 'png' )
        thumb.save( self.thumbPath, 'png' )
        
        self.image = image
        self.thumb = thumb
            
    def getImagePath( self ):
        return self.imagePath
    
    def getThumbPath( self ):
        return self.thumbPath
    
    def getImage( self ):
        return self.image
    
    def getThumb( self ):
        return self.thumb
        
    def scale( self, pixbuf, width, height ):
        pWidth, pHeight = pixbuf.get_width(), pixbuf.get_height()        
        destX = destY = 0
        destWidth, destHeight = width, height
        if pWidth > pHeight:
            scale = float( width ) / pWidth
            destHeight = int( scale * pHeight )
            destY = int( ( height - scale * pHeight ) / 2 )
        elif pHeight > pWidth:
            scale = float( height ) / pHeight
            destWidth = int( scale * pWidth )
            destX = int( ( width - scale * pWidth ) / 2 )
        else:
            return pixbuf.scale_simple( width, height, gtk.gdk.INTERP_HYPER )
        
        if scale == 1:
            return pixbuf
        
        scaled = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
        pixbuf.scale( scaled, 0, 0, width, height, \
                      destX, destY, scale, scale, \
                      gtk.gdk.INTERP_HYPER )
        return scaled
        