# -*- coding: utf-8 -*-

#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import shutil

from emesenecommon import *

# REMEMBER to increment the version number on each bunch of default config
# values you add
CONFIG_VERSION = '7'
DEFAULT_CONFIG = {
    'theme': 'default',
    'rememberMe': '1',
    'showByNick': '1',
    'showOffline': '0',
    'showEmptyGroups': '0',
    'showByStatus': '0',
    'orderByStatus': '0',
    'autoReply': '0',
    'autoReplyMessage': \
                'Im not here (this is an autoreply)'.replace( ' ', '%20' ),
    'rememberMyPassword': '0',
    'font': 'Sans',
    'color': '#000000',
    'fontSize': '12',
    'tabbedChat': '0',
    'tabPosition': str(TOP_TAB),
    'showTimestamp': '0',
    'showTabsIfOne': '0',
    'debug': '0',
    'useProxy': '0',
    'proxyHost': 'http://127.0.0.1',
    'proxyPort': '80',
    'logPath': '~',
    'convWindowWidth': '450',
    'convWindowHeight': '480',
    'mainWindowWidth': '260',
    'mainWindowHeight': '440',
    'mainWindowX': '-1',
    'mainWindowY': '-1',
    'avatarDir' : '',
    'activePlugins': 'PersonalMessage', # CSV of plugin names
    'configVersion': CONFIG_VERSION # change every time the configuration changes in emesene to know if we have to rewrite all
}

DEFAULT_COLORS = {
    'typingColor': '#339999',
    'messageWaitingColor': '#E53B1A',
    'personalMessageColor': '#AAAAAA',
    'contactBlockedColor': '#aa0000',
    'nickColor': '#000000',
    'nudge' : '#f57900',
    'doNudge' : '#fcaf3e',
    'timestamp' : '#555555',
    'linkColor' : '#FF0000'
}
        

class Config:
    '''This class contains all the stuff relative to the configuration of emesene.
    ou can get or set configuration values and will be saved to a file for persistence'''

    def __init__( self ):
        '''
        Constructor
        '''
        
        #check the operating system
        self.os = ""

        if os.name == "posix":
            self.os = "*nix"
        elif os.name == "nt" or os.name == "mac":
            self.os = "windows"
        else:
            self.os = "other" # i dont have a mac to test or the other operating systems so...

        #set the user directory        
        self.configDict = {}
        
        if self.os == "*nix":
            if not os.path.isdir( HOME_DIR + "/.config" ):
                os.mkdir( HOME_DIR + "/.config" )
            
            if not os.path.isdir( CONFIG_DIR ):
                os.mkdir( CONFIG_DIR )
                debugmsg( 'Config dir created: ' + CONFIG_DIR )
                                
                #create files
            if not os.path.isdir( THEME_HOME_PATH ):
                os.mkdir( THEME_HOME_PATH ) 
                debugmsg( 'Theme dir in config created: ' + THEME_HOME_PATH )
            
            if not os.path.isdir( PLUGIN_HOME_PATH ):
                os.mkdir( PLUGIN_HOME_PATH )
                shutil.copyfile( PLUGIN_SYSTEM_WIDE_PATH + '/__init__.py', \
                                 PLUGIN_HOME_PATH + '/__init__.py' )
                debugmsg( 'Plugins dir in config created: ' + PLUGIN_HOME_PATH )
            
            if not os.path.isdir( CACHE_PATH ):
                os.mkdir( CACHE_PATH )

            if not os.path.isfile( CONFIG_DIR + DIR_SEP + "config" ):
                self.createConfig()
                debugmsg( 'Config file created: ' + CONFIG_DIR + DIR_SEP + "config" )
                
        elif self.os == "windows":
            if not os.path.isfile( CONFIG_DIR + DIR_SEP + "config" ):
                self.createConfig()
            if not os.path.isdir( CONFIG_DIR + DIR_SEP + "cache" ):
                os.mkdir( CONFIG_DIR + DIR_SEP + 'cache' )
        
        self.readConfig()
        
        if int( self.getValue( 'configVersion', 0 ) ) < int ( CONFIG_VERSION ):
            debugmsg( 'updating config file...' )
            self.updateConfigFile( self.configDict )
            self.readConfig()
        
    def readConfig( self ):
        '''
        read the config file and create a dictionarie with key and value
        of all the key=value\n in the config file
        '''
        
        conf = None
        try:
            conf = open( CONFIG_DIR + "/config" , "r" )
            
            string = conf.read()
            string = string.replace( " " , "" )
    
            lines = string.split( "\n" )

            for i in lines:
                if i == "":
                    continue
                
                try:
                    key,value = i.split( "=" )
                    self.configDict[ key ] = value
                except Exception, e:
                    debugmsg( key + ' config value is incorrect' )
                    
            conf.close()
        except:
            if conf:
                conf.close()
            self.setDefault()
            
    def writeConfig( self ):
        '''
        write the config to the file, overwrite current config file
        '''
        
        try:
            conf = open( CONFIG_DIR + "/config" , "w" )
        
            for k, v in self.configDict.iteritems():
                conf.write( k + "=" + v + "\n" )

            conf.close()
        except Exception, e:
            errormsg( 'exception writing config:\n' )
            errormsg( e )
        
    def createConfig( self ):
        '''
        Creates a new config file
        ''' 
        self.setDefault()
        self.writeConfig()
            
    def updateConfigFile( self, actualConfig ):
        '''
        Updates the config file so it reflects all the current configuration
        parameters
        '''
        
        try:
            conf = open( CONFIG_DIR + "/config" , "w" )
            
            del actualConfig['configVersion']
            actualKeys = actualConfig.keys()
            for i in actualKeys:
                conf.write( i + "=" + actualConfig[ i ] + "\n" )
            # new values
            defaults = self.getDefaults()
            for i in defaults.keys():
                if i not in actualKeys:
                    conf.write( i + "=" + defaults[ i ] + "\n" )
                    
            conf.close()
        except Exception, e:
            errormsg( 'Exception writing config:\n' )
            errormsg( e )
                
    def getValue( self , key, default = '' ):
        '''
        get the value from key in configDict
        '''
        
        if key in self.configDict:
            return self.configDict[ key ]
        elif key in DEFAULT_CONFIG:
            return DEFAULT_CONFIG[ key ]
        elif key in DEFAULT_COLORS:
            return DEFAULT_COLORS[ key ]
        else:            
            debugmsg( 'Config value was not found: ' + key )
            return default
        
    def getDefaultValue( self, key ):
        '''
        Returns the default value for the given key
        '''
        
        if key in DEFAULT_CONFIG:
            return DEFAULT_CONFIG[ key ]
        elif key in DEFAULT_COLORS:
            return DEFAULT_COLORS[ key ]
        else:            
            debugmsg( 'Config value was not found: ' + key )
            return default

    def setValue( self , key , value ):
        '''
        set a value in configDict, to make it last to the next session
        you have to call writeConfig before you close
        '''
        
        self.configDict[ key ] = value
        self.writeConfig()
        
    def getThemes( self ):
        '''
        this method is here because i dont know where to put it,
        it return a list of dirs in the theme directory
        '''
        themes = []
        
        def explorePath( path ):
            for root, dirs, files in os.walk( path, True ):
                if root == path:
                    for dir in dirs:
                        if not dir.startswith( '.' ) and \
                                        dir not in themes:
                            themes.append( dir )
                    # stops walking the tree
                    # TODO: any other way of deleting a list's content??
                    for i in range( len( dirs ) ):
                        del dirs[ 0 ]
                    
        explorePath( THEME_HOME_PATH )
        explorePath( THEME_SYSTEM_WIDE_PATH )
        
        return themes

    def getLogPath( self ):
        if self.getValue( 'logPath' ).find( '~' ) != -1:
            if self.os == "*nix":
                return os.path.expanduser(self.getValue( 'logPath' ))
            else: #windows only donate me a mac for mac support :D
                return self.getValue( 'logPath' ).replace( '~', 'C:\\' )
        else:
            return self.getValue( 'logPath' )

    def setDefaultColors( self ):
        
        for k, v in DEFAULT_COLORS.iteritems():
            self.configDict[ k ] = v
        
    def setDefault( self ):
        '''
        set default values to the config keys, in case something fails or we
        want to reset the values...
        '''
        
        self.configDict.clear()
        
        for k, v in DEFAULT_CONFIG.iteritems():
            self.configDict[ k ] = v
        
        #emesene interface colors
        self.setDefaultColors()
        
        self.writeConfig()
        
    def getDefaults( self ):
        '''
        Returns a dictionary with all the default config vaules
        '''
        defaults = DEFAULT_CONFIG.copy()
        for k,v in DEFAULT_COLORS.iteritems():
            defaults[ k ] = v
        return defaults

