#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import sys
import time
import gobject
import gettext
import thread
import webbrowser
import traceback

import Theme
from Msnp13 import msnp13
from Msnp13 import Socket
from Msnp13 import SignalHandler
import Avatar
import Config
import Dialogs
import MainWindow
import PluginManager
import PreferenceWindow
import PluginManagerDialog
import ConversationManager
import StatusMenu
from ImageFileChooser import ImageFileChooserDialog

from emesenecommon import *

import TrayIcon
import SlashCommands

gettext.install('emesene')
gettext.install('emesene', LANG_PATH, unicode=False)
lang = {}

class Controller( object ):
    '''
    The Controller class concentrate all the logic of the program
    leaving the other classes to only implement the GUI.
    All the classes with GUI should receive a Controller instance to
    be able to do things.
    '''

    def __init__( self ):
        '''
        Contructor
        '''

        self.status_ordered = [
            [ 'NLN', 'AWY', 'BSY', 'BRB', 'PHN',
             'LUN', 'HDN', 'IDL', 'FLN' ],

            [ 'online', 'away', 'busy', 'brb', 'phone',
             'lunch', 'invisible', 'idle', 'offline' ],

            [ _( 'Online' ), _( 'Away' ), _( 'Busy' ), _( 'Be right back' ),
             _( 'On the phone' ), _( 'Gone to lunch' ), _( 'Invisible' ),
             _( 'Idle' ), _( 'Offline' ) ],
            
            [ _( '_Online' ), _( '_Away' ), _( '_Busy' ), _( 'Be _right back' ),
             _( 'On the _phone' ), _( 'Gone to _lunch' ), _( '_Invisible' ),
             _( 'I_dle' ), _( 'O_ffline' ) ]
        ]

        self.connected = False
        self.userEmail = None
        self.userStatus = None
        self.config = Config.Config()
        self.theme = Theme.Theme( self.config.getValue( "theme" ) )
        self.mainWindow = MainWindow.MainWindow( self, self.config )
        self.mainWindow.show()
        
        theme = self.theme
        gtk.window_set_default_icon_list( theme.getImage( 'icon16' ),
                                          theme.getImage( 'icon32' ),
                                          theme.getImage( 'icon48' ),
                                          theme.getImage( 'icon96' ) )

        if not TrayIcon.disabled:
            self.trayIcon = TrayIcon.TrayIcon( self )

        self.autoReplyMessage = self.config.getValue( \
            "autoReplyMessage" ).replace( "%20" , " " )
        self.conversationManager = None
        self.msn = None

        self.pluginManager = None
        
        self.trayDisconnect = None
        self.traySeparator = None
        self.trayStatusMenu = None
        
        self.avatar = None
        self.cancel = False
        self.Slash = SlashCommands.SlashCommands()
        

    def getContactManager(self):
        return (None, self.msn.contactManager)[self.msn != None]

    def login( self, user, password, status ):
        '''do the login'''
        global loginStatus
        self.cancel = False
        self.userEmail = user
        self.userStatus = status
        self.mainWindow.buildInterface( 'loading' )

        debug = self.config.getValue( 'debug' ) == '1'

        proxy = None

        if self.config.getValue( 'useProxy' ) == '1':
            proxy = Socket.Proxy( self.config.getValue( 'proxyHost' ), \
                int( self.config.getValue( 'proxyPort' ) ) )

        self.signal = SignalHandler.SignalHandler()
        self.msn = msnp13.Msnp13(  "messenger.hotmail.com", 1863, user, password,\
             self.signal, CACHE_PATH, DEFAULT_THEME_PATH + os.sep + 'userPanel.png', debug, proxy )
        
        if self.cancel == True:
            return
        
        thread.start_new_thread( login , ( self.msn , ) )

        while loginStatus == 0:
            time.sleep( 0.01 )
            if self.cancel == True:
                return
            gtk.main_iteration( True )

        if loginStatus == -1:
            Dialogs.Error( self.mainWindow , _( "Error during login, please retry" ) ).run()
            self.mainWindow.buildInterface( 'login' )
            loginStatus = 0
            return

        if self.cancel == True:
            return

        self.connected = True

        self.msn.changeStatus( status )
        self.msn.connect( 'disconnected', self.disconnected )
        gobject.timeout_add( 200 , self.process )
        self.mainWindow.buildInterface( 'userlist' )
        self.changeAvatar( self.msn.picture.getPath() )
        
        # menu items only available when connected
        if not self.trayStatusMenu and not TrayIcon.disabled:
            statusMenuItem = gtk.MenuItem( _( "_Status" ) )
            statusMenu = StatusMenu.StatusMenu( self )
            statusMenuItem.set_submenu( statusMenu )
            separator = gtk.SeparatorMenuItem()
            disconnect = gtk.ImageMenuItem( gtk.STOCK_DISCONNECT )
            disconnect.connect( 'activate', self.on_tray_disconnect )
            
            self.trayDisconnect = disconnect
            self.traySeparator = separator
            self.trayStatusMenu = statusMenuItem
        
        self.conversationManager = ConversationManager.ConversationManager( self.msn, self.theme, self.config )
        
        if not TrayIcon.disabled:
            # REMEMBER to remove them when disconnect    
            self.trayIcon.menu.prepend( self.trayDisconnect )
            self.trayIcon.menu.prepend( self.traySeparator )
            self.trayIcon.menu.prepend( self.trayStatusMenu )
            self.trayIcon.menu.show_all()
            self.trayIcon.update( self.msn.status )
        
        self.pluginManager = PluginManager.PluginManager( self, self.msn )

        plugins = self.config.getValue( 'activePlugins' ).split( ',' )

        self.startPlugins( plugins )

        self.msn.connect( 'status change', self.onStatusChange )
        self.msn.connect( 'status online', self.onStatusOnline )
        self.msn.connect( 'status offline', self.onStatusOffline )
        self.msn.connect( 'new conversation', self.newConversation )
        self.msn.connect( 'add notification', self.addNotification )
        self.msn.connect( 'remove notification', self.removeNotification )
        self.msn.connect( 'user list change', self.refreshUserList )
        self.msn.connect( 'user disconnected', self.userDisconnected )
        self.msn.connect( 'connection problem', self.connectionProblem )
        
        #self.msn.connect( 'offline message waiting', self.offlineMessageWaiting )
        #self.msn.connect( 'offline message received', self.offlineMessageReceived )

        self.checkPending()
        self.cancel = False
        
        self.msn.connect( 'slash command', self.Slash.emit )

    def disconnected( self ):
        '''called when a error ocurrs and we cant keep connected.'''
        Dialogs.Error( self.mainWindow , _( "Connection error" ) ).run()
        self.logout()
        
    def logout( self ):
        '''do the logout'''
        global loginStatus

        self.conversationManager.closeAll()

        self.mainWindow.buildInterface( 'login' )
        self.pluginManager.stopAll()
        
        self.connected = False
        
        if not TrayIcon.disabled:
            self.trayIcon.menu.remove( self.trayDisconnect )
            self.trayIcon.menu.remove( self.traySeparator )
            self.trayIcon.menu.remove( self.trayStatusMenu )
            self.trayIcon.update( self.msn.status )
            
        self.msn.logout()
        self.msn = None
        self.signal = None
        self.conversationManager = None
        self.pluginManager = None
        loginStatus = 0
        
    def cancelLogin(self):
        self.cancel = True
        self.goToLoginWindow()
        
    def goToLoginWindow( self ):
        global loginStatus
        self.mainWindow.buildInterface( 'login' )
        self.msn = None
        self.signal = None
        self.conversationManager = None
        self.pluginManager = None
        loginStatus = 0
        
    def quit( self, status ):
        '''
        close the window, and do all the things needed...
        '''
        
        # makes it more responsive to the user by effectively hiding the
        # the window before disconnecting and quitting
        self.mainWindow.connect( 'unmap-event', self.on_main_hidden, status )
        self.mainWindow.hide()
        if not TrayIcon.disabled:
            self.trayIcon.tray.hide()
        
    def on_main_hidden( self, widget, event, status ):
        if self.connected:
            self.msn.logout()

        self.mainWindow.saveToQuit()
        
        self.config.writeConfig()
        sys.exit( status )

    def process( self ):
        '''
        call all the process methods to process
        the incoming data.
        '''

        if self.connected:
            self.msn.process()
        else:
            return False

        return True

    def checkPending( self ):
        '''Check for users pending to be added'''
        users = self.msn.checkPending()
        if len( users ) > 0:
            for i in users:
                self.userAddedYouDialog(i, i)

    def startPlugins( self, plugins ):
        '''Start the plugins in pluginList'''

        if plugins[ 0 ] != '':
            for i in plugins:
                (success, message) = self.pluginManager.checkPlugin( i )
                if not success:
                    debugmsg( _( 'plugin %s could not be initialized, reason:' )%i, 3 )
                    debugmsg( message, 3 )
                else:
                    self.pluginManager.startPlugin( i )
                    debugmsg( _( 'plugin %s started successfully' )%i, 3 )

    def changeAvatarDialog( self ):
        '''
        Shows a dialog to change personal avatar
        '''
        
        response = ImageFileChooserDialog( parent=self.mainWindow, \
                                           config=self.config ).run()
        
        responseId, filename = response
        if responseId == gtk.RESPONSE_OK:
            # a image has been chosen
            self.changeAvatar( filename )
        elif responseId == RESPONSE_NO_AVATAR:
            # no avatar
            # TODO: remove avatar
            pass

    def changeAvatar( self, filename ):
        if ( filename and os.path.exists( filename ) ):
            self.avatar = Avatar.Avatar( filename, CACHE_PATH, self.msn._user )
            self.msn.setDisplayPicture( self.avatar.getImagePath() )
            self.mainWindow.setAvatar( self.avatar.getThumb() )
        else:
            Dialogs.Error( self.mainWindow , _( "File can't be found" ) ).run()

    def changeNickDialog( self, user = 'Me', errorOnEmpty=True ):
        '''
        show a dialog requesting the nick to be changed
        and change it.
        '''

        if not self.connected:
            return

        if user == 'Me':
            response = Dialogs.ChangeNick( self.mainWindow , self.msn.nick ).run()
        else:
            alias = self.msn.contactManager.getContactAlias( user )
            deleteBut = True
            if not alias:
                alias = ''
                deleteBut = False
            response = Dialogs.ChangeNick(
                                  self.mainWindow , alias, deleteBut ).run()

        if response == None:
            return
        elif errorOnEmpty and response == "":
            Dialogs.Error( self.mainWindow , _( "Empty Nickname" ) ).run()
            return

        self.changeNick( response, user )

    def changeNick( self , nick, user = 'Me' ):
        '''
        chang the nick in the session
        '''

        self.msn.changeNick( nick, user )

    def changePersonalMessage( self, pm ):
        '''
        change your personal message
        '''

        self.msn.changePersonalMessage( pm )

    def setStatus( self , status, source='' ):
        '''set the status'''

        if ( self.mainWindow.currentInterface == 'userlist' ):
            self.msn.changeStatus( status )
            self.msn.emit( 'self status changed', status )
            if not TrayIcon.disabled:
                self.trayIcon.update(self.msn.status)

    def addUserDialog( self, toGroup=None ):
        '''
        show a dialog requesting the email of the ser to be added
        '''

        if not self.connected:
            return

        response = Dialogs.AddFriend( self.mainWindow, self.msn.getGroupNames(), toGroup ).run()

        if response == None:
            return
        elif response[ 0 ] == "":
            Dialogs.Error( self.mainWindow , _( "Empty Field" ) ).run()
            return

        self.addUser( response[ 0 ], response[ 1 ] )
        self.refreshUserList()

    def addUser( self, email, group = '' ):
        '''
        add an user
        '''

        self.msn.addUser( email, group )

    def removeUser( self, email = None ):
        '''
        remove an user, if email is None the email will be the one selected
        on the user list
        '''

        if email == None:
            email = self.mainWindow.userList.getUserSelected()

        if email == "":
            Dialogs.Error( self.mainWindow , _( "Choose an user to remove" ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( "Are you sure you want to remove %s from your contact list?" )%email, \
                        gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeUser( email )

        self.refreshUserList()

    def blockUser( self, email = None ):
        '''
        block an user, if email is None the email will be the one selected
        on the user list
        '''

        if email == None:
            user = self.mainWindow.userList.getUserSelected()

            if user == "":
                Dialogs.Error( self.mainWindow , _( "Choose an user" ) ).run()
                return

            self.msn.blockUser( user )

        else:
            self.msn.blockUser( email )
        
        self.refreshUserList()

    def unblockUser( self, email = None ):
        '''
        unblock an user, if email is None the email will be the one selected
        on the user list
        '''

        if email == None:
            user = self.mainWindow.userList.getUserSelected()

            if user == "":
                Dialogs.Error( self.mainWindow , _( "Choose an user" ) ).run()
                return

            self.msn.unblockUser( user )

        else:
            self.msn.unblockUser( email )

        self.refreshUserList()

    def addGroupDialog( self ):
        '''
        show a dialog requesting the name of the group to be added
        '''

        if not self.connected:
            return

        response = Dialogs.AddGroup( self.mainWindow ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self.mainWindow , _( "Empty Field" ) ).run()
            return

        self.addGroup( response )
        self.refreshUserList()

    def addGroup( self, group ):
        '''
        add a group
        '''

        self.msn.addGroup( group )

    def addNotification( self, command, tid, params, payload ):
        '''
        this method is called when a user adds you
        '''

        # this looks like when someone adds you
        # <ml><d n="gmail.com"><c n="luismarianoguerra" t="1" l="8" f="[Mariano%20Guerra]%20(IRC%20is%20just%20multiplayer%20notepad.)" /></d></ml>

        email = payload.split( '<c n="' )[ 1 ].split( '"' )[ 0 ] + '@' + payload.split( '<ml><d n="' )[ 1 ].split( '"' )[ 0 ]
        nick = ''

        try:
            nick = payload.split( ' f="' )[ 1 ].split( '"' )[ 0 ].replace( '%20', ' ' )
        except:
            pass

        self.userAddedYouDialog(nick, email)

    def userAddedYouDialog( self, nick, email ):
        d = Dialogs.AddBuddy( self.mainWindow , nick + " (" + email + ")"  )
        response = d.run()

        if response == gtk.RESPONSE_ACCEPT:
            self.addUser(email)
        else:
            pass # TODO make the user that adds you not able to see you!!

    def removeNotification( self, command, tid, params, payload ):
        '''
        this method is called when a user removes you
        '''

        # this looks like when someone removes you:
        # <ml><d n="gmail.com"><c n="luismarianoguerra" t="1" l="8" /></d></ml>

        #mail = payload.split( '<c n="' )[ 1 ].split( '"' )[ 0 ] + '@' + payload.split( '<ml><d n="' )[ 1 ].split( '"' )[ 0 ]
        #Dialogs.Information( self.mainWindow , _( 'The user ' ) + mail + _( ' has deleted you' ) ).run()
        pass

    def removeGroup( self, group = None ):
        '''
        remove a group, if group is None the group will be the selected on the userlist
        '''

        if group == None:
            group = self.mainWindow.userList.getGroupSelected()

        if group == "":
            Dialogs.Error( self.mainWindow , _( "Choose a group to delete" ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( "Are you sure you want to delete %s?" )%group, \
            gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeGroup( group )

        self.refreshUserList()

    def renameGroupDialog( self, group = None ):
        '''
        show a dialog requesting the new name of the group, if group is None
        use the selected group in the userlist
        '''

        if group == None:
            group = self.mainWindow.userList.getGroupSelected()

            if group == "":
                Dialogs.Error( self.mainWindow , _( "Choose a group" ) ).run()
                return

        response = Dialogs.RenameGroup( self.mainWindow , group ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self.mainWindow , _( "Empty group name" ) ).run()
            return

        self.renameGroup( group, response )
        self.refreshUserList()

    def renameGroup( self, oldName, newName ):
        '''
        rename a group from oldName to newName
        '''

        self.msn.renameGroup(oldName, newName)

    def preferencesDialog( self ):
        '''
        show a preference dialog
        '''

        p = PreferenceWindow.PreferenceWindow( self, self.config, self.mainWindow )
        p.show()

    def themeChanged( self ):
        '''called when the theme changes'''

        theme = self.config.getValue( "theme" )
        result = self.theme.setTheme( theme )
        if not result:
            Dialogs.Error( self.mainWindow, \
                       _( "Error loading theme, falling back to default" ) \
                       ).run()
            self.config.setValue( 'theme', 'default' )
            theme = 'default'

        self.mainWindow.rebuild()
        if self.trayStatusMenu:
            self.trayStatusMenu.set_submenu( StatusMenu.StatusMenu( self ) )
            self.trayIcon.menu.show_all()
            
        return theme

    def setAutoReplyDialog( self ):
        '''
        show a dialog requesting the autoreply message
        '''

        if not self.connected:
            return

        response = Dialogs.SetAutoReply( self.mainWindow , self.autoReplyMessage ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self , _( "Empty AutoReply" ) ).run()
        else:
            self.autoReplyMessage = response
            self.config.setValue( "autoReplyMessage" , response.replace( " " , "%20" ) )

    def showUserData( self, data ):
        '''
        show the user nick or mail in the status bar
        '''
        self.mainWindow.showUserData( data )

    def refreshUserList( self ):
        '''
        call to refreshUserList on MainWindow that calls refresh on userList
        userList get the values of showOffline etc from config, so if you want
        to change something, change it on config.
        '''

        self.mainWindow.refreshUserList()

    # signal handlers
    def connectionProblem( self ):
        '''called when a PNG is not ansered'''
        
        Dialogs.Error( self.mainWindow, \
                       _( "Connection  error" ) \
                       ).run()
                       
        self.logout()

    def onStatusChange( self, command, tid, params ):
        '''
        callback for the status change event
        '''
        self.refreshUserList()

    def onStatusOnline( self, command, tid, params ):
        '''
        callback for the status online event
        '''
        self.refreshUserList()

    def onStatusOffline( self, command, tid, params ):
        '''
        callback for the status offline event
        '''
        self.refreshUserList()
        
    def on_tray_disconnect( self, *args ):
        self.logout()

    def updateTabPositionIfTabbedChat(self):
        if self.conversationManager != None:
            if self.config.getValue( 'tabbedChat' ) == '1' and \
                    len( self.conversationManager.conversations ) > 0:
                window = self.conversationManager.conversations.itervalues().next()[ 0 ]
                window.setTabPosition(int(self.config.getValue('tabPosition')))

    def newConversation( self, mail, switchboard = None ):
        '''
        This method is called when the user want to initiate a new conversation with someone,
        or when a switchboard has been created and we want to give a window to it.
        (callback for 'new conversation' signal emmited by msnp13)
        '''

        self.conversationManager.newConversation( mail, switchboard )

    def pluginDialog( self ):
        '''
        Show the pluginManagerDialog
        '''

        p = PluginManagerDialog.PluginManagerDialog( self.pluginManager, self.mainWindow, self.config )
        p.show()

    def seeSpace( self, email ):

        self.msn.getSpaceData( email )
        #webbrowser.open( 'http://spaces.live.com/' + user )

    def offlineMessageWaiting( self, msnOIM ):
        '''process the OIM messages'''

        for i in msnOIM.getMessages():
            msnOIM.retrieve( i[ 'I' ] )

    def offlineMessageReceived( self, soapResponse, msnOIM ):
        '''process the OIM message'''

        message = ''

        try:
            data = msnOIM.getMessage( soapResponse.getBody() )
            message = data.split( '\n\n' )[ 1 ].split( '\n' )[ 0 ]
        except Exception, e:
            print 'exception handling offline message'
            print str( e )

        fromMail = msnOIM.getSender( soapResponse.getBody() )
        message = msnOIM.decodeMessage( message )

        Dialogs.Information( self.mainWindow, 'Offline message received:\nFrom: ' + fromMail + '\nMessage: ' + message ).run()

    def userDisconnected( self, tid, params ):
        '''method called when the server disconnect us'''

        Dialogs.Error( self.mainWindow , _( 'The server has disconnected you.' ) ).run()
        self.logout()

    def getGroups( self ):
        '''return the list of groups'''

        return self.msn.getGroups()
    
    def getGroupNames( self, ordered=True):
        '''
        Returns a list with the group names in the contact list
        '''
        
        groups = self.msn.getGroupNames()
        groups.sort( key= lambda x: x.lower() )
        
        return groups

    def moveUserToGroup( self, user, oldGroup, group ):
        '''move a user to a group'''

        self.msn.moveUserToGroup( user, oldGroup, group )

    def addUserToGroup( self, user, group ):
        '''add a user to a group'''

        if group == 'No group':
            return

        self.msn.addUserToGroup( user, group )

    def removeUserFromGroup( self, user, group ):
        '''remove a user from a group'''

        self.msn.removeUserFromGroup( user, group )
        
    def getMenuData( self ):
        return self.mainWindow.userList.getMenuData()

loginStatus = 0

def login( msn ):
    global loginStatus
    try:
        msn.login()
    except Exception, e:
        traceback.print_exc()
        loginStatus = -1
        print e
        return False
    except:
        loginStatus = -1
        return False

    loginStatus = 1
    return False

if __name__ == '__main__':
    path = sys.path[0]
    
    originalPath = os.getcwd()
    os.chdir( path )
    sys.path.append(path)

    controller = Controller()

#    try:
#        controller = Controller()
#    except Exception, e:
#        errormsg('Couldn\'t initialize:\n')
#        errormsg( e )
#        sys.exit( -1 )
    gtk.main()
    
    os.chdir( originalPath )
