# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk
import pango

import Conversation
import ConversationWindow

class ConversationManager( object ):
    '''
    This class handle a collection of conversations
    '''

    def __init__( self, msn, theme, config ):
        '''
        Contructor
        '''

        self.msn = msn
        self.theme = theme
        self.config = config

        self.textTagTable = gtk.TextTagTable()

        tag = gtk.TextTag(name='nudge')
        tag.set_property( 'pixels-above-lines', 6 )
        tag.set_property( 'pixels-below-lines', 6 )
        tag.set_property( 'left-margin', 40 )
        tag.set_property( 'right-margin', 50 )
        tag.set_property( 'foreground-gdk', \
            gtk.gdk.color_parse( self.config.getValue('nudge') ) )
        tag.set_property( 'style', pango.STYLE_ITALIC )

        self.textTagTable.add( tag )
        
        tag = gtk.TextTag(name='donudge')
        tag.set_property( 'pixels-above-lines', 4 )
        tag.set_property( 'pixels-below-lines', 4 )
        tag.set_property( 'left-margin', 40 )
        tag.set_property( 'right-margin', 50 )
        tag.set_property( 'foreground-gdk', \
            gtk.gdk.color_parse( self.config.getValue('doNudge') ) )
        tag.set_property( 'style', pango.STYLE_ITALIC )
        
        self.textTagTable.add( tag )

        tag = gtk.TextTag(name='tstamp')
        tag.set_property( 'left-margin', 15 )
        tag.set_property( 'foreground-gdk', \
            gtk.gdk.color_parse( self.config.getValue('timestamp') ) )
        tag.set_property( 'style', pango.STYLE_ITALIC )

        self.textTagTable.add( tag )

        tag = gtk.TextTag(name='sender')
        tag.set_property( 'pixels-above-lines', 6 )
        tag.set_property( 'pixels-below-lines', 1 )
        tag.set_property( 'weight', pango.WEIGHT_BOLD )
        tag.set_property( 'scale', 1.1 * pango.SCALE_MEDIUM )

        self.textTagTable.add( tag )

        tag = gtk.TextTag(name='me')
        tag.set_property( 'pixels-above-lines', 6 )
        tag.set_property( 'pixels-below-lines', 1 )
        tag.set_property( 'weight', pango.WEIGHT_BOLD )
        tag.set_property( 'scale', 1.1 * pango.SCALE_MEDIUM )

        self.textTagTable.add( tag )

        tag = gtk.TextTag(name='link')
        tag.set_property( 'foreground-gdk', \
            gtk.gdk.color_parse( self.config.getValue('linkColor') ) )
        tag.set_property( 'underline', pango.UNDERLINE_SINGLE )

        self.textTagTable.add( tag )

        #a dict of tuples assigned by email key
        #eg.: "jhon@ta.com" : (ConversationWindow, Conversation)
        self.conversations = {}

    def newConversation( self, mail, switchboard ):
        '''
        Create a new conversation, add it to the dict with the mail and return it
        if switchboard is None we create the switchboard
        '''
        contactStatus = self.msn.contactManager.getContactStatus(mail)
        windowIcon = self.theme.statusToPixbuf(contactStatus)
        
        # remove the closed windows
        for i in self.conversations.keys():
            if self.conversations[ i ][ 0 ].isClosed() or \
                ( self.conversations[ i ][ 0 ].notOpen and \
                    self.conversations[ i ][ 1 ].getStatus() == 'closed' ):
                del self.conversations[ i ]
            elif self.conversations[ i ][ 1 ].isClosed():
                del self.conversations[ i ]

        if not self.conversations.has_key( mail ):
            wasNone = (switchboard == None)
            if switchboard == None:
                switchboard = self.msn.newSwitchboard()
                switchboard.invite( mail )
                
                switchboard.getDisplayPicture( mail )

            conversation = Conversation.Conversation( switchboard, self.theme,\
                 self.config, self.textTagTable )

            if self.config.getValue( 'tabbedChat' ) == '1' and \
                len( self.conversations ) > 0:
                    window = self.conversations.itervalues().next()[ 0 ]
                    window.openTab( conversation, mail )
                    window.set_icon(windowIcon)

                    if wasNone:
                        window.show()

                    self.conversations[ mail ] = ( window, conversation )
                    conversation.setWindow( window )
            else:
                self.conversations[ mail ] = (ConversationWindow.ConversationWindow( \
                    self.msn, conversation, mail, self.theme, self.config ), conversation)
                window = self.conversations[mail][0]
                window.set_icon(windowIcon)
                conversation.setWindow(window)

                if wasNone:
                    self.conversations[ mail ][ 0 ].show()
        else:
            if switchboard != None:
                self.conversations[ mail ][ 1 ].setSwitchboard( switchboard )
            else:
                self.conversations[ mail ][ 0 ].show()

            self.conversations[ mail ][ 0 ].present()

        return self.conversations[ mail ][ 0 ]

    def closeAll( self ):
        '''
        close all the conversations
        '''

        for (key,value) in self.conversations.iteritems():
            value[ 0 ].hide()

        self.conversations = {}
