# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import gobject

import ListOfUsers
import SmilieWindow
from emesenecommon import escape
import pango
import time

from math import sqrt

from htmltextview import HtmlTextView

class ConversationUI( gtk.VBox ):
    '''this class represent all the widgets that are inside a tab
    also old the tab widget because there is no better place than
    this to hold it...'''
    
    def __init__( self, conversation, config, theme ):
        gtk.VBox.__init__( self )
        self.conversation = conversation
        self.config = config
        self.theme = theme
        
        self.header = Header( conversation, config, self.theme )
        self.tabWidget = TabWidget( conversation, config )
        self.input = InputWidget(conversation, theme, config)
        self.toolbar = ToolbarWidget(conversation, theme, config, self.input)
        self.listOfUsers = ListOfUsers.ListOfUsers( self.conversation.getMembersDict(), self.theme )
        
        self.scrollList = gtk.ScrolledWindow()
        self.scrollList.set_shadow_type(gtk.SHADOW_IN)
        self.scrollList.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scrollList.set_shadow_type( gtk.SHADOW_IN )
        self.scrollList.set_size_request( 180, -1 )
        self.scrollList.add( self.listOfUsers )
        
        self.textview = HtmlTextView( self.conversation.getTextBuffer() )
        #self.textview = gtk.TextView( self.conversation.getTextBuffer() )
        self.textview.set_wrap_mode( gtk.WRAP_WORD )
        self.textview.set_left_margin( 6 )
        self.textview.set_right_margin( 6 )
        self.textview.set_editable( False )
        self.textview.set_cursor_visible( False )

        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        self.scroll.add( self.textview )
        
        paned = gtk.HPaned()
        paned.pack1( self.scroll, True, False )
        paned.pack2( self.scrollList, False, True )
        
        paned.connect( 'map', self.on_paned_map )
        
        self.paned = paned
        
        self.pack_start( self.header, False, False )
        self.pack_start( self.paned, True, True )
        self.pack_start( self.toolbar, False, True, 3 )
        self.pack_start( self.input, False, True, 3 )
        
        #self.input.grabFocus()
        self.show_all()
        self.update()


    def on_paned_map( self, *args ):
        rect = self.paned.get_allocation()
        # Golden ratio ;)
        listWidth = int( rect.width * 2 / (3 + sqrt(5)) )
        self.scrollList.set_size_request( listWidth, -1 )
        
    def update( self ):
        self.header.update()
        self.tabWidget.update()
        self.listOfUsers.populate( self.conversation.getMembersDict() )
        
        if len( self.conversation.getMembers() ) > 1:
            self.showUserList()
        else:
            self.hideUserList()
        
    def showUserList( self ):
        self.scrollList.show()
        
    def hideUserList( self ):
        self.scrollList.hide()
        
    def getTabWidget( self ):
        return self.tabWidget
    
    def getTextView( self ):
        return self.textview
    
    def setCloseTabCallback( self, callback ):
        self.tabWidget.setCloseTabCallback( callback )
        
    def setMessageWaiting( self ):
        self.tabWidget.setMessageWaiting()
        
    def setTyping( self ):
        self.header.setTyping()
        self.tabWidget.setTyping()
        
    def setDefault( self ):
        self.tabWidget.setDefault()
        
    def getInput(self):
        return self.input
        
class Header( gtk.HBox ):
    '''the header of the conversation'''
    
    def __init__( self, conversation, config, theme ):
        '''Contructor'''
        
        gtk.HBox.__init__( self )
        
        self.conversation = conversation
        self.config = config
        self.theme = theme
        
        self.set_border_width( 6 )
        self.set_spacing( 6 )
        
        self.statusImage = gtk.Image()
        self.statusImage.set_from_pixbuf( self.theme.getImage( 'groupChat' ) )
        
        self.label = gtk.Label()
        self.label.set_alignment( 0.0, 0.5 )
        self.label.set_line_wrap( True )
        self.label.set_ellipsize( pango.ELLIPSIZE_END )
        
        self.label.set_markup( '<b><i>' + _( 'Connecting...' ) + '</i></b>' )
        
        self.pack_start( self.statusImage, False, False )
        self.pack_start( self.label )
        self.show_all()
        
    def update( self ):
        self.setLabel()
        
        members = self.conversation.getMembersDict()
        if len( members ) == 1:
            contact = members.values()[ 0 ]
            self.statusImage.set_from_pixbuf( self.theme.statusToPixbuf( contact.getStatus() ) )
        else:
            self.statusImage.set_from_pixbuf( self.theme.getImage( 'userPanel' ) )
    
    def setTyping( self ):
        typingColor = self.config.getValue('typingColor','#339999')
        styleOpen = "<b><span foreground='" + typingColor + "'>"
        styleClose = "</span></b>"
        self.setLabel( styleOpen, styleClose )
        gobject.timeout_add( 5000 , self.clearTyping )
        
    def clearTyping( self ):
        self.setLabel()
    
    def setLabel( self, styleOpen='<b>', styleClose='</b>' ):
        members = self.conversation.getMembersDict()
        
        personalMessageColor = self.config.getValue('personalMessageColor','#AAAAAA')
        
        if len( members ) == 1:
            contact = members.values()[ 0 ]
            self.text = styleOpen + escape(contact.getNick()) + styleClose
            
            if len( contact.getPersonalMessage() ) > 0:
                self.text += "\n<span foreground='" + personalMessageColor + "'>"
                self.text += escape( contact.getPersonalMessage() ) + "</span>"
        
            self.text += "\n<span foreground='" + personalMessageColor + "'>"
            self.text += contact.getEmail() + "</span>"
        elif len( members ) > 1:
            self.text = styleOpen + _( 'Group chat' ) + styleClose
            self.text += "\n<span foreground='" + personalMessageColor + "'>"
            self.text += str( len( members ) ) + ' ' + _( 'Members' ) + "</span>"

            self.statusImage.set_from_pixbuf( self.theme.getImage( 'groupChat' ) )
        else:
            self.text = styleOpen + '<i>' + _( 'Connecting...' ) + '</i>' + styleClose
                
        self.label.set_markup( self.text )
          
    def setDefaultLabel( self ):
        self.setLabel()
        
class TabWidget( gtk.HBox ):
    '''this class represent the widget that is inside the tab
    it contains a label and a close button'''
    
    def __init__( self, conversation, config ):
        '''Contructor'''
        
        gtk.HBox.__init__( self )
        self.conversation = conversation
        self.config = config
        
        self.text = self.conversation.getTitle()
        self.messageWaiting = False
        
        self.label = gtk.Label( self.text )
        self.label.set_use_markup( True )
        
        img = gtk.Image()
        img.set_from_stock( gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU )
        
        self.closeButton = gtk.Button()
        self.closeButton.set_image( img )
        self.closeButton.set_relief( gtk.RELIEF_NONE )
        
        self.pack_start( self.label, True, True )
        self.pack_start( self.closeButton, False,False )
        self.show_all()
        
        self.closeCallback = None
        self.closeButton.connect( 'clicked', self.onCloseClicked )
        
    def onCloseClicked( self, *args ):    
        if self.closeCallback != None:
            self.closeCallback( self.conversation, self.conversation.tabNum )
            
    def setCloseCallback( self, callback ):
        self.closeCallback = callback
        
    def update( self ):
        title = self.conversation.getTitle()
        if not title == '':
            self.text = title
        
        if self.messageWaiting:
            self.setMessageWaiting()
        else:
            self.setDefault()
    
    def setDefault( self ):
        self.label.set_markup( self.text )
        self.messageWaiting = False
        
    def setTyping( self ):
        typingColor = self.config.getValue('typingColor', '#339999')
        self.label.set_markup( '<span foreground="' + typingColor + '">' + \
            escape(self.text) + '</span>' )
        gobject.timeout_add( 5000 , self.clearTyping )
        
    def clearTyping( self ):
        if self.messageWaiting:
            self.setMessageWaiting()
        else:
            self.setDefault()
        #return False to not be called again
        return False
    
    def setMessageWaiting( self ):
        messageWaitingColor = self.config.getValue( 'messageWaitingColor', '#E53B1A' )
        self.label.set_markup( '<span foreground="' + messageWaitingColor + '">' + \
            escape(self.text) + '</span>' )
        self.messageWaiting = True
    
    def setCloseTabCallback( self, callback ):
        self.closeCallback = callback
    
class InputWidget(gtk.HBox):
        """This class represents the input widgets: text entry, sendo button and clear button
        """
        def __init__(self, conversation, theme, config):
            gtk.HBox.__init__(self)
            self.conversation = conversation
            self.theme = theme
            self.config = config
            
            self.lastKeyPressed = 0
            
            self.image = gtk.Image()
            self.image.set_from_pixbuf( self.theme.getUserDisplayPicture( self.conversation.switchboard.firstUser ) )
            self.conversation.switchboard.msn.connect( 'display picture changed', self.displayPictureChanged )
            self.pack_start(self.image, False, False, 0)
            
            self.input = gtk.TextView()
            self.input.set_left_margin(6)
            self.input.set_right_margin(6)
            self.input.set_wrap_mode(gtk.WRAP_WORD)
            
            self.inputBuffer = self.input.get_buffer()
            
            self.scrollInput = gtk.ScrolledWindow()
            self.scrollInput.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.scrollInput.add( self.input )
            
            frameInput = gtk.Frame()
            frameInput.add(self.scrollInput)
            
            self.pack_start(frameInput, True, True, 0)
            
            sendButton = gtk.Button(stock = gtk.STOCK_OK)
            sendButton.connect("pressed", self.on_sendButton_pressed)
            sendButton.connect("released", self.on_sendButton_released)
            
            clearButton = gtk.Button(stock = gtk.STOCK_CLEAR)
            clearButton.connect("released", self.on_clearButton_released)
            
            inputButtonVbox = gtk.VBox()
            inputButtonVbox.pack_start(sendButton, True, True, 2)
            inputButtonVbox.pack_start(clearButton, True, True, 2)
            
            self.pack_start(inputButtonVbox, False, False, 5)
            
            self.inputBuffer.connect( 'changed', self.on_changed_event )
            self.input.connect( "key-press-event" , self.on_key_press_event )
            self.input.connect( "key-release-event" , self.on_key_release_event )
            
            self.input.connect_after( 'map-event', self.on_input_map_event )
            self.input.grab_focus()
            self.tag = None
            self.show_all()
        
        def grabFocus(self):
            self.input.grab_focus() 
            
        def getInput(self):
            return self.input
        
        def getBuffer(self):
            return self.inputBuffer
        
        def on_input_map_event( self, widget, event ):
            self.input.grab_focus()
        
        def on_sendButton_released(self, *args):
            return self.clearMessageInput()

        def on_sendButton_pressed(self, *args):
            return self.sendMessage()
    
        def on_clearButton_released(self, *args):
            return self.clearMessageInput()
        
        def clearMessageInput(self):
            self.setInputText("")
            return True
        
        def sendMessage(self):
            if self.getInputText() == '':
                    return True
    
            message = self.getInputText()
            self.conversation.sendMessage( message )
            self.setInputText( '' )
            return True
        
        def setInputText( self , string , tag = None ):
            if tag == None:
                self.inputBuffer.set_text( string )
            else:
                pass
            
        def getInputText( self ):
            '''
            return the text in the input textview
            '''
            return self.inputBuffer.get_text( self.inputBuffer.get_start_iter(),\
                 self.inputBuffer.get_end_iter() , True )
            
        def appendInputText( self, text, tag = None ):
            '''
            append the given text to the inputBuffer, if tag != None inset with the given tag
            '''
            self.inputBuffer.insert_at_cursor( text )
            
        def on_changed_event( self, *args ):
            '''
            method called when text is inserted in the textbuffer
            '''
    
            # sents user typing notification
            actualTime = int( time.time() )
    
            if ( actualTime - self.lastKeyPressed ) > 5:
                self.lastKeyPressed = actualTime
    
                try:
                    self.conversation.sendIsTyping()
                except Exception , e:
                    print str( e )
                    
            self.updateInputFormat()
                    
        def applyAttrsToInput( self ):
            '''apply the current attributes to the text in input'''
    
            self.tag = self.conversation.getTextTag()
            self.inputBuffer.get_tag_table().add( self.tag )
            self.tag.set_property( 'left-margin', 6 )
            self.tag.set_priority( self.inputBuffer.get_tag_table().get_size() - 1 )
            self.inputBuffer.apply_tag( self.tag,
                                                self.inputBuffer.get_start_iter(),
                                                self.inputBuffer.get_end_iter() )
            
            self.conversation.switchboard.msn.emit( 'input format changed', self.input )
            
        def updateInputFormat(self):
            if self.tag != None:
                self.inputBuffer.apply_tag( self.tag,
                                                self.inputBuffer.get_start_iter(),
                                                self.inputBuffer.get_end_iter() )
            
            self.conversation.switchboard.msn.emit( 'input format changed', self.input )
            
        def on_key_press_event( self , widget, event ):
            if not( event.state & gtk.gdk.SHIFT_MASK ) and \
                not( event.state & gtk.gdk.CONTROL_MASK ) and \
                not( event.state & gtk.gdk.MOD1_MASK ) and \
                ( event.keyval == gtk.keysyms.Return or
                  event.keyval == gtk.keysyms.KP_Enter ):
    
                return self.sendMessage()
            
            elif event.keyval == gtk.keysyms.Escape:
                self.conversation.close()
    
        def on_key_release_event( self , widget, event ):
            if not( event.state & gtk.gdk.MODIFIER_MASK ) and \
                ( event.keyval == gtk.keysyms.Return or
                  event.keyval == gtk.keysyms.KP_Enter ):
    
                return self.clearMessageInput()
            
        def displayPictureChanged( self, email ):
            if self.conversation.switchboard.firstUser == email:
                self.image.set_from_pixbuf( self.theme.getUserDisplayPicture( email ) )
            
class ToolbarWidget(gtk.Toolbar):
    """This represents a toolbar that contains text formatation, smilies buttom, do nudge button,
    and so on...
    """
    
    def __init__(self, conversation, theme, config, input):
        gtk.Toolbar.__init__(self)
        self.conversation = conversation
        self.theme = theme
        self.config = config
        
        self.smilieWindow = SmilieWindow.SmilieWindow( theme, \
                                                       self.smilieSelected, \
                                                       None )

        self.input = input

        self.boldButton = gtk.ToggleToolButton()
        self.boldButton.set_stock_id(gtk.STOCK_BOLD)
        self.boldButton.connect( 'toggled', self.toggled, 'bold' )

        self.italicButton = gtk.ToggleToolButton()
        self.italicButton.set_stock_id(gtk.STOCK_ITALIC)
        self.italicButton.connect( 'toggled', self.toggled, 'italic' )

        self.underlineButton = gtk.ToggleToolButton()
        self.underlineButton.set_stock_id(gtk.STOCK_UNDERLINE)
        self.underlineButton.connect( 'toggled', self.toggled, 'underline' )

        smilie = gtk.ToolButton()
        smilie.set_label( _( 'Smilie' ) )
        imgSmilie = gtk.Image()
        imagenGrin = theme.getSmiley( ":D" )
        
        if type( imagenGrin ) == gtk.gdk.PixbufAnimation:
            imgSmilie.set_from_pixbuf( imagenGrin.get_static_image() )
        elif type( imagenGrin ) == gtk.gdk.Pixbuf:
            imgSmilie.set_from_pixbuf( imagenGrin )
        else:
            imgSmilie.set_from_stock( gtk.STOCK_MISSING_IMAGE ,gtk.ICON_SIZE_SMALL_TOOLBAR )
            print ':D image is not a gtk.pixbuf or pixbufanimation this should not happen please fix me!'
            print 'the type is ' + str( type( imagenGrin ) ) 
            
        smilie.set_icon_widget( imgSmilie )

        smilie.connect( 'clicked', self.showSmilieWindow )

        nudge = gtk.ToolButton()
        nudge.set_label( _( 'Nudge' ) )
        imgNudge = gtk.Image()
        imgNudge.set_from_pixbuf( theme.getImage( 'nudge' ) )
        nudge.set_icon_widget( imgNudge )

        nudge.connect( 'clicked', self.doNudge )

        invite = gtk.ToolButton()
        invite.set_label( _( 'Invite' ) )
        invite.set_stock_id( gtk.STOCK_ADD )

        invite.connect( 'clicked', self.showInviteDialog )
        
        #clear conversation toolbar button
        imgclear = gtk.Image()
        imgclear.set_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_LARGE_TOOLBAR )
        clear = gtk.ToolButton( imgclear, _('Clear Conversation') )

        clear.connect( 'clicked', self.clearOutputText )

        # Add buttons to the toolbar
        self.insert( self.boldButton, -1 )
        self.insert( self.italicButton, -1 )
        self.insert( self.underlineButton, -1 )
        
        self.insert( gtk.SeparatorToolItem(), -1 )
        
        self.insert( smilie, -1 )
        self.insert( nudge, -1 )

        self.insert( gtk.SeparatorToolItem(), -1 )

        self.insert( invite, -1 )
        self.insert(clear, -1)

        # Set tooltips to buttons
        tooltips = gtk.Tooltips()
        tooltips.enable()

        self.boldButton.set_tooltip( tooltips, _( 'Bold' ) )
        self.italicButton.set_tooltip( tooltips, _( 'Italic' ) )
        self.underlineButton.set_tooltip( tooltips, _( 'Underline' ) )
        smilie.set_tooltip( tooltips, _( 'Insert a smilie' ) )
        nudge.set_tooltip( tooltips, _( 'Send nudge' ) )
        invite.set_tooltip( tooltips, _( 'Invite a friend to the conversation' ) )
        clear.set_tooltip( tooltips, _( 'Clear conversation' ) )
        
        self.show_all()
        
    def toggled( self, button, option ):
        '''
        callback for the toggled signal
        '''

        if option == 'bold':
            self.conversation.setBold( button.get_active() )
        elif option == 'italic':
            self.conversation.setItalic( button.get_active() )
        elif option == 'underline':
            self.conversation.setUnderline( button.get_active() )

        self.conversation.ui.getInput().applyAttrsToInput()
        
    def smilieSelected( self, smilie=None ):
        '''
        this method is called when the user click a smilie in the smiliewindow
        '''

        if smilie:
            self.input.appendInputText( smilie )
        self.input.grabFocus()
        
    def showSmilieWindow( self, *args ):
        '''
        this method is called when the user click the smilie button
        '''

        self.smilieWindow.show()
        
    def showInviteDialog( self, *args ):
        '''
        this method is called when the user click the invite button
        '''

        dictionary = self.conversation.contactManager.getOnlineUsersDict()
        members = {}
        
        for i in self.conversation.getMembers() + [ self.conversation.getUser() ]:
            members[ i ] = self.conversation.contactManager.getContact( i )
            
        InviteWindow( None, dictionary,\
          members, self.theme,\
          self.conversation.inviteUser ).run()
          
    def clearOutputText(self, *args):
        self.conversation.getTextBuffer().set_text('')
        
    def doNudge( self, *args ):
        '''
        this method is called when the user click the nudge button
        '''

        self.conversation.doNudge()
        
    def setBold( self, value ):
        self.boldButton.set_active( value )
        
    def setItalic( self, value ):
        self.italicButton.set_active( value )
        
    def setUnderline( self, value ):
        self.underlineButton.set_active( value )
    
          
class InviteWindow( gtk.Dialog ):
    '''
    This class represent a list where the user can pick wich users he want to
    invite to a conversation
    '''

    def __init__( self, father, onlineUsers, usersInConversation, theme, callback ):
        '''
        Contructor
        callaback is called when a user is selected with double click
        '''

        gtk.Dialog.__init__( self, _( 'Invite' ), father,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                     (gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT) )

        self.set_default_size( 200, 200 )
        self.callback = callback
        users = onlineUsers
        
        for i in usersInConversation.keys():
            if users.has_key( i ):
                del users[ i ]

        self.userList = ListOfUsers.ListOfUsers( users, theme, self.userListClicked )
        
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy( gtk.POLICY_AUTOMATIC , gtk.POLICY_AUTOMATIC )
        self.scroll.add( self.userList )
        
        self.connect( 'response', self.invite )
        self.connect( 'delete-event', self.close )

        self.vbox.pack_start( self.scroll )
        self.vbox.show_all()
        
    def invite( self, dialog, response_id ):
        '''
        get the user in the userlist and call the callback
        '''

        try:
            if response_id == gtk.RESPONSE_ACCEPT:
                self.callback( self.userList.getSelected() )
                self.close()
                return True
            else:
                return False
        except Exception, e:
            print e
            
    def userListClicked( self, email ):
        self.callback( email )
        self.close()

    def close( self, *args ):
        '''
        close the window
        '''

        self.hide()
