# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import struct
import urllib
import md5

import Msnobj
import Switchboard

#------------------------- FROM HERE -----------------------------
# Copyright 2005 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details
_PRODUCT_KEY = 'O4BG@C7BWLYQX?5G'
_PRODUCT_ID = 'PROD01065C%ZFN6F' 
MSNP11_MAGIC_NUM = 0x0E79A9C1

def doChallenge(chlData):
    md5digest = md5.md5(chlData + _PRODUCT_KEY).digest()

    # Make array of md5 string ints
    md5Ints = struct.unpack("<llll", md5digest)
    md5Ints = [(x & 0x7fffffff) for x in md5Ints]

    # Make array of chl string ints
    chlData += _PRODUCT_ID
    amount = 8 - len(chlData) % 8
    chlData += "".zfill(amount)
    chlInts = struct.unpack("<%di" % (len(chlData)/4), chlData)

    # Make the key
    high = 0
    low = 0
    i = 0
    while i < len(chlInts) - 1:
        temp = chlInts[i]
        temp = (MSNP11_MAGIC_NUM * temp) % 0x7FFFFFFF
        temp += high
        temp = md5Ints[0] * temp + md5Ints[1]
        temp = temp % 0x7FFFFFFF

        high = chlInts[i + 1]
        high = (high + temp) % 0x7FFFFFFF
        high = md5Ints[2] * high + md5Ints[3]
        high = high % 0x7FFFFFFF

        low = low + high + temp

        i += 2

    high = littleEndify((high + md5Ints[1]) % 0x7FFFFFFF)
    low = littleEndify((low + md5Ints[3]) % 0x7FFFFFFF)
    key = (high << 32L) + low
    key = littleEndify(key, "Q")

    longs = [x for x in struct.unpack(">QQ", md5digest)]
    longs = [littleEndify(x, "Q") for x in longs]
    longs = [x ^ key for x in longs]
    longs = [littleEndify(abs(x), "Q") for x in longs]
    
    out = ""
    for x in longs:
        x = hex(x)
        x = x[2:-1]
        x = x.zfill(16)
        out += x.lower()
    
    return out

def littleEndify(num, c="L"):
    return struct.unpack(">" + c, struct.pack("<" + c, num))[0]
# ------------------------------ TO HERE ----------------------------------


class DefaultSignalHandler:
    '''
    This class represent the default signal handler for msnp13
    you should inherit from this class, if you know what you are doing
    you can create your own base class without inheriting from this one
    '''
    
    def __init__( self, msn ):
        '''
        contructor
        '''
        
        self.msn = msn
        self._user = self.msn._user
        self._debug = self.msn._debug
        self._proxy = self.msn._proxy

        self.socket = self.msn.socket
        
    def challenge( self, command, tid, params ):
        '''
        reply a challenge made from the server
        '''
        
        out = doChallenge( params )
        self.socket.sendPayloadCommand( 'QRY' , _PRODUCT_ID , out )
                      
    def statusChange( self, command, tid, params ):
        '''
        handle a status change
        '''
        global flag
        #ILN 9 IDL jingilmicio@yahoo.com .%20J%20i%20N%20G%20. 1342554148 %3Cmsnobj%20Creator%3D%22jing_il_micio%40yahoo.it%22%20Size%3D%225637%22%20Type%3D%223%22%20Location%3D%22TFR10B.dat%22%20Friendly%3D%22AAA%3D%22%20SHA1D%3D%22X0qXqwshIMyNnCGFYDEmOrLfFrE%3D%22%20SHA1C%3D%22CTsCEDGiY82DrNMYGVJAj01x%2F%2BA%3D%22%2F%3E

        t = params.split( " " )
        status = t[ 0 ]
        email = t[ 1 ].lower()
        nick = urllib.unquote(t[2])
        
        self.msn.contactManager.setContactStatus( email, status )
        self.msn.contactManager.setContactNick( email, nick )
        
        if len( t ) == 5:
            msnobj = t[ 4 ]
            contact = self.msn.contactManager.getContact( email )
            if contact != None:
                msnObject = Msnobj.createFromString( msnobj )
                if contact.getMsnobj() == None or\
                (msnObject != None and\
                contact.getMsnobj().quote() != msnobj):
                    contact.setMsnobj( msnObject )
                    self.msn.emit( 'msnobj changed', msnObject )     
               
        if self.msn.contactManager.getContactNick( email ) != nick:
            self.msn.emit( 'nick changed', email, nick )     
        
        self.msn.emit( 'contact status change', email, status )   
        
        self.socket.debug( "Status Change: " + email + " is " + status )
    
    def statusOnline( self, command, tid, params ):
        '''
        handle a friend going online
        '''
        
        status = tid
        t = params.split(' ')
        email = t[ 0 ].lower()
        
        nick = urllib.unquote(t[1])
        
        oldStatus = self.msn.contactManager.getContactStatus( email )
	
        self.msn.contactManager.setContactStatus( email, status )
        
        if len( t ) == 4:
            msnobj = t[ 3 ]
            contact = self.msn.contactManager.getContact( email )
            if contact != None:
                msnObject = Msnobj.createFromString( msnobj )
                if contact.getMsnobj() == None or\
                (msnObject != None and\
                contact.getMsnobj().quote() != msnobj):
                    contact.setMsnobj( msnObject )
                    self.msn.emit( 'msnobj changed', msnObject )
        
        if self.msn.contactManager.getContactNick( email ) != nick:
            self.msn.emit( 'nick changed', email, nick )
            
        self.msn.contactManager.setContactNick( email, nick )
        self.socket.debug('FRIEND %s (%s) changed status from: %s to :%s:' % \
            (nick, email, oldStatus, status))
        self.msn.emit( 'user online', email, oldStatus )
        self.msn.emit( 'contact status change', email, status )   
    
    def statusOffline( self, command, tid, params ):
        '''
        handle a friend going offline
        '''
        
        email = tid.lower()
        self.msn.contactManager.setContactStatus( email, 'FLN' )
        self.socket.debug('FRIEND %s disconnected' % (email,))
        self.msn.emit( 'user offline', email )
    
    def switchboardInvitation( self, command, tid, params, msnp13 ):
        '''
        handle the switchboard invitation, this is the first message
        that a buddy send to you when he wants to chat with you
        '''
        #example:
        # RNG 146980 207.46.2.154:1863 CKI 1138138395.4459 example@passport.com Example%20Name U messenger.hotmail.com
        
        completeCommand = command + ' ' + tid + ' ' + params
        if len( completeCommand.split( ' ' ) ) != 9:
            print 'invalid message in switchboardInvitation, not opening switchboard'
            return
        
        ( command, sessionID, host, authenticationType, authenticationString, user, userName, u, server ) = completeCommand.split( ' ' )
        user = user.lower()
        msnp13.switchboards[ user ] = Switchboard.Switchboard( tid, msnp13, 'invited' )
        msnp13.switchboards[ user ].setConnectionString( completeCommand )
        
    def serverMessage( self, command, tid, params, payload ):
        '''called when we receive a MSG in the main socket'''
        
        if payload.find( 'Content-Type: text/x-msmsgsinitialmdatanotification;' ) == -1:
            return
        
        if payload.find( 'Mail-Data: ' ) == -1:
            return
        
        data = payload.split( 'Mail-Data: ' )[ 1 ].split( '\r\n' )[ 0 ]
        self.msn.msnOIM.parseMailData( data )

        if self.msn.msnOIM.getMessageCount() > 0:
            self.msn.emit( 'offline message waiting', self.msn.msnOIM )
