# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import base64

from uuid import uuid4
from string import Template
import XmlTemplates
import DefaultSignalHandler
import SoapManager
from xml.dom.minidom import parseString

class ResponseHandler(object):
    def __init__(self):
        self.status=500

    def handle(self,response):
        dom = parseString(response.getBody())
        ret = dom.getElementsByTagName('soap:Fault')
        if(ret.length>0):
            raise 'OIMError',ret[0]

class MsnOIM (object):
    '''
    This is the class to handle msn oim
    '''

    def __init__(self,msn):
        self.msn=msn
        self._lockKey=''
        self._passportid=''
        self._seq=1
        self._t=''
        self._p=''

        self.responseHandler=ResponseHandler()
        self.msg_list=[]

        self.debug=msn.socket.debug

    def getMessageCount( self ):
        '''Return the number of messages waiting'''
        
        return len( self.msg_list )
    
    def getMessages( self ):
        '''Return the msg_lst'''
        
        return self.msg_list

    def send(self,to,msg):
        '''Send an offline message to friend'''

        soapTemplate=Template(XmlTemplates.send_message)
        nick='=?%s?%s?data?=%s?=' % ('utf-8','B',self.msn.nick.encode('base64').strip())
        
        d=dict(memberName=self.msn._user,friendlyName=nick,ver='MSNP13',buildVer='8.0.0792',to=to,passport=self._passportid.replace('&','&amp;'),appid=DefaultSignalHandler._PRODUCT_ID,lockKey=self._lockKey,seqNum=self._seq,runId=uuid4().__str__(),content=msg.encode('base64').strip())
        self._seq=self._seq+1

        ret = self.soapRequest('http://messenger.msn.com/ws/2004/09/oim/Store','Send OIM','ows.messenger.msn.com',443,'/OimWS/oim.asmx',soapTemplate, d)
        return ret!=None

    def parseMailData(self,mailData):
        ''' Parse the mail data xml'''
        mailDataDom=parseString(mailData)
        for m in mailDataDom.getElementsByTagName('M'):
            item={}
            for node in m.childNodes:
                if node.nodeType!=node.TEXT_NODE and node.childNodes.length==1:
                    item[node.nodeName]=node.firstChild.nodeValue
            self.msg_list.append(item)        
        return False

    def retrieve(self,message_id):
        '''retreive an oim message by message_id'''

        if(self._passportid!=''):
            s=self._passportid[2:]
            self._t,self._p=s.split('&p=')

        soapTemplate=Template(XmlTemplates.retreive_message)
        d=dict(t=self._t,p=self._p,mid=message_id)

        self.soapRequest('http://www.hotmail.msn.com/ws/2004/09/oim/rsi/GetMessage','Retreive OIM','rsi.hotmail.com',443,'/rsi/rsi.asmx',soapTemplate,d)
        self.delete( message_id )

    def getMessage( self, xmlString ):
        '''Return the message from the xml'''
        
        dom=parseString(xmlString)
        return dom.getElementsByTagName('GetMessageResult')[0].firstChild.nodeValue

    def getSender( self, xml ):
        '''return the From of the message'''

        # From: =?utf-8?B?V2FyIFsgd3d3LmlnbmFjaW9ndWVycmEuY29tLmFyIF0gWyBDZWx1bGFyIG51ZXZv?= &lt;iogua@gmail.com&gt;
        try:
            return xml.split( 'From: =?' )[ 1 ].split( '&lt;' )[ 1 ].split( '&gt;' )[ 0 ]
        except:
            return '?'

    def decodeMessage( self, message ):
        '''return the message decoded'''
        
        return base64.b64decode( message )

    def delete(self,message_id):
        '''Delete an oim message'''
        if(self._passportid!=''):
            s=self._passportid[2:]
            self._t,self._p=s.split('&p=')

        soapTemplate=Template(XmlTemplates.delete_message)
        d=dict(t=self._t,p=self._p,mid=message_id)

        self.soapRequest('http://www.hotmail.msn.com/ws/2004/09/oim/rsi/DeleteMessages','Delete OIM','rsi.hotmail.com',443,'/rsi/rsi.asmx',soapTemplate,d)

    def soapRequest(self,url,description,server,port,path,soapTemplate,d):
        '''Soap request method handle Tweener challenge and Lock key challenge'''
        try:
            body=soapTemplate.safe_substitute(d)
            request = SoapManager.SoapRequest( url, server, port, path, body, self.handle )
            SoapManager.put( request )
        except 'OIMError',xml:
            retry=False
            # TweenerChanllenge:
            if(xml.getElementsByTagName('TweenerChallenge').length>0):
                # we get the hash
                hash = xml.getElementsByTagName('TweenerChallenge')[0].firstChild.nodeValue
                self.debug('Tweener Challenge:' + hash)
                # start the passport authentication
                self._passportid = self.msn.passportAuth( hash )
                # Reset d
                s=self._passportid[2:]
                self.debug('New passport:'+self._passportid)
                self._t,self._p=s.split('&p=')
                d['passport']=self._passportid.replace('&','&amp;')
                d['t']=self._t
                d['p']=self._p
                retry=True

            # LockKeyChallenge:    
            if(xml.getElementsByTagName('LockKeyChallenge').length>0):
                hash = xml.getElementsByTagName('LockKeyChallenge')[0].firstChild.nodeValue
                self.debug('LockKey Challenge:' + hash)
                self._lockKey=DefaultSignalHandler.doChallenge(str(hash))
                # Reset d
                d['lockKey']=self._lockKey
                self.debug('New lockKey:'+ d['lockKey'])
                retry =True
            
            if retry:
                self.soapRequest(url,description,server,port,path,soapTemplate,d)

            self.debug(xml.toxml())
        except Exception, e:
            self.debug('OIM Exception, which is not documented!')
            self.debug( str( e ) )
    
    def handle( self, response ):
        self.responseHandler.handle( response )
        self.msn.emit( 'offline message received', response, self )
        
