# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# FOR INFORMATION ABOUT THIS PROTOCOL GO TO:
# msnpiki.msnfanatic.com
# siebe.bot2k3.net/docs/

import os
import sys
import struct
import random
import base64

from exceptions import IndexError

import Msnobj

class P2PManager( object ):
    
    def __init__( self, switchboard ):
        self.switchboard = switchboard
        
        self.senders = []
        self.receivers = []
        
    def process( self, message, mail ):
        response = None
        
        for i in self.senders + self.receivers:
            if i.isMessage( message ):
                response = i.process( message )
                break
        #TODO: erase finished senders/receivers
        if response == None and message.find( 'INVITE MSNMSGR:' ) != -1:
            s = Sender( self.switchboard.msn.getUser(), mail, message, self.switchboard.msn.picture )
            r = s.process( s.message )
            
            for j in r:
                self.switchboard.sendMessage(j, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + mail )
        elif response != None:
            for j in response:
                self.switchboard.sendMessage(j, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + mail )
    
    def newReceiver( self, To, From, msnobj, filename ):
        receiver = Receiver( To, From, msnobj, filename, self.switchboard.msn.emit ) 
        self.receivers.append( receiver )
        return receiver
            
class Base( object ):
    STATUS_CREATED = 0 # when just created
    STATUS_INVITE = 1
    STATUS_200OK = 2
    STATUS_PREPARATION = 3
    STATUS_SENDING = 4
    STATUS_RECEIVING = 5
    STATUS_FINISH = 6
    STATUS_END = 7 # when all the stuff is finished
    
    CONTENT_LIMIT = 1202
    
    CONTENT_INVITE = 'application/x-msnmsgr-sessionreqbody'
    CONTENT_BYE = 'application/x-msnmsgr-sessionclosebody'
    CONTENT_CLOSE = 'application/x-msnmsgr-sessionclosebody'
    
    #flags

    #* 0x0 when no flags are specified
    NO_FLAG = int( 0x0 )
    #* 0x2 if it's an acknowledgement
    ACK_FLAG = int( 0x2 )
    #* 0x8 if there was an error on the binary level
    BINERROR_FLAG = int( 0x8 )
    #* 0x20 when the message content is data for display pictures or custom emoticons
    PIC_OR_EMO_FLAG = int( 0x20 )
    #* 0x01000030 if it's data of a file 
    FILE_FLAG = int( 0x01000030 )

    EUFGUID_DISPLAY = '{A4268EEC-FEC5-49E5-95C3-F126696BDBF6}'
    EUFGUID_FILE = '{5D3E02AB-6190-11D3-BBBB-00C04F795683}'

    FOOTER = '\0\0\0\0'
    DP_FOOTER = '\0\0\0\1'
    FILE_FOOTER = '\0\0\0\2'
    
    def __init__( self ):
        self.status = Base.STATUS_CREATED
        
        self.From = ''
        self.To = ''
        self.Via = ''
        self.CSeq = ''
        self.CallID = ''
        self.MaxForwards = ''
        self.ContentType = ''
        self.ContentLength = ''
        self.EUFGUID = ''
        self.SessionID = ''
        self.AppID = ''
        self.Context = ''
        self.lastHeader = None
        self.msnobj = None
        
        self.picture = None
        self.identifier = self.generateRandomNumber( 50000 )
        self.baseID = self.identifier # baseID will be modified
        # we keep identifier because we need to skip the current value
                
    def setPicture( self, value ):
        self.picture = value

    def getPicture( self ):
        return self.picture
    
    def setLastHeader( self, value ):
        self.lastHeader = value

    def getLastHeader( self ):
        return self.lastHeader

    def setMsnobj( self, value ):
        self.msnobj = value

    def getMsnobj( self ):
        return self.msnobj
        
    def setFrom( self, value ):
        self.From = value

    def getFrom( self ):
        return self.From

    def setTo( self, value ):
        self.To = value

    def getTo( self ):
        return self.To

    def setVia( self, value ):
        self.Via = value

    def getVia( self ):
        return self.Via

    def setCSeq( self, value ):
        self.CSeq = value

    def getCSeq( self ):
        return self.CSeq

    def setCallID( self, value ):
        self.CallID = value

    def getCallID( self ):
        return self.CallID

    def setMaxForwards( self, value ):
        self.MaxForwards = value

    def getMaxForwards( self ):
        return self.MaxForwards

    def setContentType( self, value ):
        self.ContentType = value

    def getContentType( self ):
        return self.ContentType

    def setContentLength( self, value ):
        self.ContentLength = value

    def getContentLength( self ):
        return self.ContentLength

    def setEUFGUID( self, value ):
        self.EUFGUID = value

    def getEUFGUID( self ):
        return self.EUFGUID

    def setSessionID( self, value ):
        self.SessionID = value

    def getSessionID( self ):
        return self.SessionID

    def setAppID( self, value ):
        self.AppID = value

    def getAppID( self ):
        return self.AppID

    def setContext( self, value ):
        self.Context = value

    def getContext( self ):
        return self.Context
        
    def setStatus( self, status ):
        self.status = status
        
    def getStatus( self ):
        return self.status
    
    def nextBaseID( self ):
        if self.baseID == self.identifier:
            self.baseID -= 3
        else:
            self.baseID += 1
            
            if self.baseID == self.identifier:
                self.baseID += 1
    
    def generateRandomNumber( self, min = 0, max = sys.maxint ):
        return random.randint( min, max )
     
    def randomid( self ):
        id = '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        
        return id
    
    def getParameters( self, s ):
        '''parse a message and create and return a dict with the values'''
        d = {}
        try:
            header = s.split( '\r\n\r\n' )[ 1 ][ :48 ]
            d[ 'header' ] = header
        except:
            pass
        
        for i in s.split( '\r\n' ):
            l = i.split( ': ' )
            
            if len( l ) == 2:
                d[ l[ 0 ] ] = l[ 1 ]
                
        return d
    
    def printParameters( self, s ):
        '''print the name and values of the parameters on a message'''
        
        for ( key, value ) in s.iteritems():
            if key == 'header':
                continue
            
            if len( key ) > 20:
                fill = ': '
            else:
                fill = ': ' + ' ' * (20 - len( key ))
                
            print key +  fill + str( value )
            
    def getMessageType( self, message ):
        """receive a message as a parameter and return the
        type of message that it is.
        returned types are:
        * Invitation
        * 200 OK Message
        * Data Preparation Message
        * Data Message
        * BYE Message
        * Ack
        * None if there were problems"""
        
        params = self.getParameters( message )
        
        if not params.has_key( 'header' ):
            print '\nNO HEADER IN getMessageType!\n'
            return None
        
        header = BinHeader( params[ 'header' ] )
        
        if self.isAck( header ):
            # its hard to know wich ack is so we let
            # other method with more information to decide
            return 'Ack'
        else:
            if self.contain( message, 'INVITE MSNMSGR:' ):
                return 'Invitation'
            elif self.contain( message, 'MSNSLP/1.0 200 OK' ):
                return '200 OK Message'
            elif header.getSessionID() != 0 and header.getTotalDataSize() == 4:
                return 'Data Preparation Message'
            elif header.getSessionID() != 0 and header.getFlag() == Base.PIC_OR_EMO_FLAG:
                return 'Data Message'
            elif self.contain( message, 'BYE MSNMSGR:' ):
                return 'BYE Message'
    
    def isAck( self, header ):
        return header.getAcknowledgedIdentifier() > 0

    def contain( self, string, substring ):
        return string.find( substring ) != -1
        
class Sender( Base ):

    def __init__( self, To, From,inviteMessage, picture ):
        Base.__init__( self )
        
        self.setTo( To )
        self.setFrom( From )
        self.picture = picture
        self.message = inviteMessage
        
        if inviteMessage.find( "INVITE MSNMSGR:" ) == -1:
            print "Not an invite message"
            self.setStatus( Base.STATUS_FINISH )
        
        via = self.extractViaBranch()
        if via == "":
            print "Malformed invitation finishing, no Via ID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.Via = via
            
        callID = self.extractCallID()
        
        if callID == "":
            print "Malformed invitation finishing, no Call-ID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.setCallID( callID )
            
        sid = self.extractSessionID()
        
        if sid == "":
            print "Malformed invitation finishing, no SessionID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.setSessionID( sid )
            
            
    #Via: MSNSLP/1.0/TLP ;branch={D7B851BF-97F2-47C4-8009-11FAFABDE1A9}
    def extractViaBranch( self ):
        """try to extract the via id from the invite message return empty string otherwise """        
        try:
            return self.message.split( "Via: MSNSLP/1.0/TLP ;branch={" )[ 1 ].split( "}" )[ 0 ]
        except:
            return ""
            
    def extractCallID( self ):
        """try to extract the callID from the invite message return empty string otherwise """        
        try:
            return self.message.split( "Call-ID: {" )[ 1 ].split( "}" )[ 0 ]
        except:
            return ""
            
     #SessionID: \r\n
    def extractSessionID( self ):
        """try to extract the sessionID from the invite message return an empty string otherwise"""
        try:
            return self.message.split( "SessionID: " )[ 1 ].split( "\r\n" )[ 0 ]
        except:
            return ""
     
    def process( self, message ):
    
        params = self.getParameters( message )
        
        try:
            header = BinHeader( params[ 'header' ] )
        except:
            print "NO HEADER IN MESSAGE!"
            return []
            
        if self.getStatus() == Base.STATUS_CREATED:
            self.setStatus( Base.STATUS_FINISH )
            
            try:
                return self.sendBaseID( header, params ) + self.send200OK() + \
                self.sendDataPreparation() + self.sendData()
            except Exception, e:
                print str( e )
                return []
        else:
            return []
        
    def isMessage( self, message ):
        return False
    
    def sendBaseID( self, bin, params ):
        try:
            self.SessionID = int( params[ 'SessionID' ] )
        except:
            raise Exception( 'Could not get sessionID from invite' )
        self.hisBaseID = bin.getIdentifier()
            
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(bin.getTotalDataSize())
        header.setAcknowledgedDataSize(bin.getTotalDataSize())
        header.setFlag(Base.ACK_FLAG)
        header.setAcknowledgedIdentifier(bin.getIdentifier())
        header.setAcknowledgedUniqueID(bin.getAcknowledgedIdentifier())
        
        ack = header.getData() + Base.FOOTER
        self.lastHeader = header
        return [ ack ]
    
    def send200OK( self ):
        content = 'SessionID: ' + str( self.SessionID ) + '\r\n\r\n\0'
            
        s = 'MSNSLP/1.0 200 OK\r\n'
        s += 'To: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.Via + '}\r\n'
        s += 'CSeq: 1 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: ' + Base.CONTENT_INVITE + '\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
    
        self.lastHeader = header
        ack200 = header.getData() + s
    
        self.setStatus(Base.STATUS_INVITE)
        return [ ack200 ]
    
    def sendDataPreparation( self ):
        header = BinHeader()
        header.setSessionID( self.SessionID )
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(4)
        header.setMessageLength(4)
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.lastHeader = header
        
        return [ header.getData() + '\0\0\0\0' + Base.FOOTER ]
        
    
    def sendData( self ):
        if not self.picture or self.picture.getSize() == 0:
            raise Exception( 'picture size 0 aborting' )

        moreData = True
        l = []
        
        while moreData:
            header = BinHeader()
            header.setSessionID( self.SessionID )
            header.setIdentifier( self.baseID )
            header.setDataOffset( self.picture.getPointer() )
            header.setTotalDataSize( self.picture.getSize() )
            header.setFlag( Base.PIC_OR_EMO_FLAG )
            header.setAcknowledgedIdentifier( self.generateRandomNumber() )
            
            if (self.picture.getSize() - self.picture.getPointer() ) > Base.CONTENT_LIMIT:
                header.setMessageLength( Base.CONTENT_LIMIT )
                chunk = self.picture.getData()[ self.picture.getPointer():self.picture.getPointer() + Base.CONTENT_LIMIT ] 
                self.picture.setPointer( self.picture.getPointer() + Base.CONTENT_LIMIT ) 
            else:
                header.setMessageLength( (self.picture.getSize() - self.picture.getPointer()) )
                chunk = self.picture.getData()[ self.picture.getPointer(): ]
                self.picture.setPointer( self.picture.getSize() )
                moreData = False
            
            self.lastHeader = header    
            l.append( header.getData() + chunk + Base.DP_FOOTER )
            
        return l
        
class Receiver( Base ):

    def __init__( self, To, From, msnobj, filename, emitCb ):
        Base.__init__( self )
        
        self.setSessionID( self.generateRandomNumber( min=50000, max=99999 ) )
        self.setTo( To )
        self.setFrom( From ) # from its me
        self.setMsnobj( msnobj )
        self.setVia( self.randomid() )
        self.setCallID( self.randomid() )
        self.setPicture( Picture( filename, Picture.CREATE ) )
        
        self.emitCb = emitCb
        
        self.hisBaseID = 0
        self.hisLastHeader = None
        self.nextMessageIdWaited = 0
    
    def compare( self, header ):
        print "\nstatus: " + str( self.getStatus() )
        print "identifiers: " + str( self.identifier ) + " =? " + str(header.getAcknowledgedIdentifier())
        print "TDS: " + str( self.lastHeader.getTotalDataSize() ) + "=?" + str( header.getAcknowledgedDataSize() )
        print "SID: " + str(self.lastHeader.getSessionID()) +  "==" +  str( header.getSessionID() ) + "\n"
        
    def process( self, message ):
        params = self.getParameters( message )
        
        if not params.has_key( 'header' ):
            print '\n' + self.To + ' NO HEADER IN PROCESS!\n'
            return []
        
        header = BinHeader( params[ 'header' ] )
        
        if message.find( "Conn-Type: Direct-Connect" ) != -1 and \
        message.find( 'INVITE MSNMSGR:' ) != -1:
            print "direct connect, sending decline"
            return self.decline()
            
        if self.getStatus() == Base.STATUS_INVITE and \
        self.identifier == header.getAcknowledgedIdentifier():
            self.hisBaseID = header.getIdentifier()
            self.hisLastHeader = header
            return []
        elif self.getStatus() == Base.STATUS_INVITE and \
        self.contain( message, 'MSNSLP/1.0 200 OK' ):
            self.hisLastHeader = header
            return self.ack200OK( header )
        elif self.getStatus() == Base.STATUS_INVITE:
            print 'not a baseid message or a 200 ok on Receiver.process, closing'
            self.setStatus( Base.STATUS_END )
            return []
        elif self.getStatus() == Base.STATUS_200OK:
            self.hisLastHeader = header
            return self.DpmAck( header )
        elif self.getStatus() == Base.STATUS_PREPARATION:
            if header.getTotalDataSize() == 0:
                print 'totaldatasize == 0 on Receiver.process, closing'
                self.setStatus( Base.STATUS_END )
                return []
            else:
                self.hisLastHeader = header
                return self.receiveData( message, header )
        else:
            print 'Unknown message on Receiver.process, closing'
            self.setStatus( Base.STATUS_END )
            return []

    def isMessage( self, message ):
        params = self.getParameters( message )
        
        if not params.has_key( 'header' ):
            print '\n' + self.To + ' NO HEADER IN PROCESS!\n'
            return False
        
        header = BinHeader( params[ 'header' ] )
        
        #if self.hisLastHeader != None:
        #    print "nextMessageIdWaited : " + str(self.nextMessageIdWaited)
        #    print "got : " + str(header.getIdentifier())
        
        if message.find( "Conn-Type: Direct-Connect" ) != -1 and \
        message.find( 'INVITE MSNMSGR:' ) != -1:
            return True
        
        if self.getStatus() == Base.STATUS_CREATED:
            print "\njust created return false\n"
            return False
        if self.getStatus() == Base.STATUS_INVITE and \
        self.identifier == header.getAcknowledgedIdentifier() and \
        self.lastHeader.getTotalDataSize() == header.getAcknowledgedDataSize() and \
        header.getFlag() == Base.ACK_FLAG:
            print "identified invite " + str( self )
            return True
        elif self.getStatus() == Base.STATUS_INVITE and \
        self.contain( message, 'MSNSLP/1.0 200 OK' ) and \
        self.hisLastHeader != None and \
        header.getIdentifier() == self.hisLastHeader.getIdentifier() - 3:
            self.nextMessageIdWaited = self.hisLastHeader.getIdentifier() - 2
            print "identified invite 1 " + str( self )
            return True
        elif self.getStatus() == Base.STATUS_200OK and \
        self.nextMessageIdWaited == header.getIdentifier():
            self.nextMessageIdWaited = self.hisLastHeader.getIdentifier() - 1
            print "identified 200ok " + str( self )
            return True
        elif self.getStatus() == Base.STATUS_PREPARATION and \
        self.getSessionID() == header.getSessionID():
            print "identified preparation " + str( self )
            return True
        else:
            print "not from " + str( self )
            return False
        
    def inviteMessage( self ):
        self.setStatus(Base.STATUS_INVITE)
        
        # a message from gaim 2.0.6 to WLM that works
        '''
        INVITE MSNMSGR:xmxsxn@hotmail.com MSNSLP/1.0
        To: <msnmsgr:xmxsxn@hotmail.com>
        From: <msnmsgr:luismarianoguerra@gmail.com>
        Via: MSNSLP/1.0/TLP ;branch={CE68F97B-2977-4899-AAA0-7277B172B781}
        CSeq: 0 
        Call-ID: {89BCFC9D-408B-41C2-823C-E60601891369}
        Max-Forwards: 0
        Content-Type: application/x-msnmsgr-sessionreqbody
        Content-Length: 344

        EUF-GUID: {A4268EEC-FEC5-49E5-95C3-F126696BDBF6}
        SessionID: 158288
        SChannelState: 0
        AppID: 12
        Context: PG1zbm9iaiBDcmVhdG9yPSJ4bXhzeG5AaG90bWFpbC5jb20iIFNpemU9IjE1MzUxIiBUeXBlPSIzIiBMb2NhdGlvbj0iVEZSMkMyLnRtcCIgRnJpZW5kbHk9IkFBQT0iIFNIQTFEPSJ6V0VlNlB3T0JBSDBnQ0lxbVZVUjJycHpOR2M9IiBTSEExQz0iT3pqdEdQZHNuUkJqVVpSaDBIT3V5T0RvbENBPSIvPgA=

        \0
        '''
        
        content = 'EUF-GUID: {A4268EEC-FEC5-49E5-95C3-F126696BDBF6}\r\n'
        content += 'SessionID: ' + str( self.getSessionID() ) + '\r\n'
        content += 'SChannelState: 0\r\nAppID: 12\r\n'
        content += 'Context: ' + base64.b64encode( str( self.msnobj  ) + '\0' ) + '\r\n\r\n\0'
        
        s = 'INVITE MSNMSGR:' + self.getTo() + ' MSNSLP/1.0\r\n'
        s += 'To: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.getVia() + '}\r\n'
        s += 'CSeq: 0 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: ' + Base.CONTENT_INVITE + '\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.lastHeader = header
        
        return header.getData() + s
    
    def decline( self ):
        content = 'SessionID: ' + str( self.SessionID ) + '\r\n\r\n\0'
        
        s = 'MSNSLP/1.0 603 Decline\r\n'
        s += 'To: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.Via + '}\r\n'
        s += 'CSeq: 1 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: application/x-msnmsgr-sessionreqbody\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
    
        self.lastHeader = header
            
        self.setStatus(Base.STATUS_FINISH)
        return [ header.getData() + s ]
        
    def basicAck( self, bin ):
        '''return a basic ack header'''
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(bin.getTotalDataSize())
        header.setAcknowledgedDataSize(bin.getTotalDataSize())
        header.setFlag(Base.ACK_FLAG)
        header.setAcknowledgedIdentifier(bin.getIdentifier())
        header.setAcknowledgedUniqueID(bin.getAcknowledgedIdentifier())
        
        return header
        
    def ack200OK( self, bin ):
        self.setStatus(Base.STATUS_200OK)
        header = self.basicAck( bin )
        self.lastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def DpmAck( self, bin ):
        self.setStatus(Base.STATUS_PREPARATION)
        header = self.basicAck( bin )
        self.lastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def receiveData( self, message, bin ):
        self.picture.setPointer( bin.getDataOffset() + bin.getMessageLength() )
        self.picture.appendData( message.split( '\r\n\r\n' )[ 1 ][ 48:48 + bin.getMessageLength() ] )
            
        print 'actual ' + str( self.picture.getPointer() ) + ' size ' +  str( bin.getTotalDataSize() )
            
        if self.picture.getPointer() == bin.getTotalDataSize():
            print 'writing file..'
            self.picture.write()
            
            if self.msnobj.getType() == "3":
                self.emitCb( 'display picture changed', self.To )
            elif self.msnobj.getType() == "2":
                self.emitCb( 'custom emoticon transfered', self.To, self.picture.getPath() )
                
            self.setStatus( Base.STATUS_FINISH )
            
            return self.ackData( bin ) + self.bye( bin )
        
        return []
            
    def ackData( self, bin ):
        header = self.basicAck( bin )
        header.setSessionID( self.getSessionID() )
        self.lastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def bye( self, bin ):
        s = 'BYE MSNMSGR:' + self.getTo() + ' MSNSLP/1.0\r\n'
        s += 'To: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.randomid() + '}\r\n'
        s += 'CSeq: 0 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: application/x-msnmsgr-sessionclosebody\r\n'
        s += 'Content-Length: 3\r\n'
        s += '\r\n\r\n\0\0\0\0\0'
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setFlag( 0x80 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.lastHeader = header
        
        return [ header.getData() + s ]

class Picture( object ):
    '''a class that contains data about our display picture'''
    
    OPEN = 1
    CREATE = 2
    
    def __init__( self, path, action ):
        '''Contructor'''
        
        self.path = path
        
        if action == Picture.OPEN:
            f = file( path, 'r' )
            if f:
                self.data = f.read()
                f.close()
            else:
                self.data = ''
        else:
            self.data = ''
        
        self.size = len( self.data )
        #point to the actual point where the data is
        #being readed
        self.pointer = 0
        
    def write( self ):
        f = file( self.path, 'w' )
        f.write( self.data )
        f.close()
        
    def getData(self):
        return self.data
    
    def setData( self, data ):
        self.data = data
        
    def appendData( self, data ):
        self.data += str( data )
        
    def getPointer( self ):
        return self.pointer
    
    def setPointer( self, position ):
        self.pointer = position
        
    def getSize( self ):
        return self.size
    
    def setSize( self, size ):
        self.size = size
        
    def getPath( self ):
        return self.path
    
    def setPath( self, path ):
        self.path = path
       
class ByteArray( object ):
    '''a class that represent a byte array (little endian)'''
    
    def __init__( self, size ):
        '''create a byte array with size=size and initialize with 0'''
        
        self.fmt = '<' + str( size ) + 'B'
        self.data = struct.pack( self.fmt, *[ 0 for i in range( size ) ] )
        self.valueList = list( struct.unpack( self.fmt, self.data ) )

    def getData( self ):
        return self.data

    def __setitem__( self, pos, value ):
        '''set the value in pos'''
        
        if len( self.valueList ) > pos:
            self.valueList[ pos ] = value
            self.data = struct.pack( self.fmt, *self.valueList )
            
    def __getitem__( self, pos ):
            return self.valueList[ pos ]
        
    def setDword( self, pos, value ):
        # less significative
        byte1 = self.__extractByte( value, 0 )
        byte2 = self.__extractByte( value, 1 )
        byte3 = self.__extractByte( value, 2 )
        byte4 = self.__extractByte( value, 3 )
        
        self[ pos ] = byte1
        self[ pos + 1 ] = byte2
        self[ pos + 2 ] = byte3
        self[ pos + 3 ] = byte4
        
    def getDword( self, pos ):
        byte1 = self[ pos ]
        byte2 = self[ pos + 1 ]
        byte3 = self[ pos + 2 ]
        byte4 = self[ pos + 3 ]
        
        return (byte4<<24) + (byte3<<16) + (byte2<<8) + byte1
        
    def setQword( self, pos, value ):
        byte1 = self.__extractByte( value, 0 )
        byte2 = self.__extractByte( value, 1 )
        byte3 = self.__extractByte( value, 2 )
        byte4 = self.__extractByte( value, 3 )
        
        byte5 = self.__extractByte( value, 4 )
        byte6 = self.__extractByte( value, 5 )
        byte7 = self.__extractByte( value, 6 )
        byte8 = self.__extractByte( value, 7 )
        
        self[ pos ] = byte1
        self[ pos + 1 ] = byte2
        self[ pos + 2 ] = byte3
        self[ pos + 3 ] = byte4
        
        self[ pos + 4 ] = byte5
        self[ pos + 5 ] = byte6
        self[ pos + 6 ] = byte7
        self[ pos + 7 ] = byte8
        
    def getQword( self, start ):
        return (self.getDword( start + 4 ) << 32) + self.getDword( start )
    
    def __extractByte( self , number , byte ):
        '''
        extract a given byte of a number and return it
        0 -> first byte ( less significative )
        '''
        
        return ( number & ( 255 << byte * 8 )) >> ( byte * 8 )
        
    def __repr__( self ):
        return self.data
    
    def __len__(self):
        return len( self.valueList )

       
class BinHeader( ByteArray ):
    '''this class represent the bin header'''
    
    def __init__( self, data = None ):
        '''Contructor'''
        
        ByteArray.__init__( self, 48 )
        
        if data != None:
            self.fill( data )
            
    def setSessionID( self, dword ):
        self.setDword( 0, dword )
        
    def getSessionID( self ):
        return self.getDword( 0 )
    
    def setIdentifier( self, dword ):
        self.setDword( 4, dword )
        
    def getIdentifier( self ):
        return self.getDword( 4 )
    
    def setDataOffset( self, qword ):
        self.setQword( 8, qword )
        
    def getDataOffset( self ):
        return self.getQword( 8 )
    
    def setTotalDataSize( self, qword ):
        self.setQword( 16, qword )
        
    def getTotalDataSize( self ):
        return self.getQword( 16 )
    
    def setMessageLength( self, dword ):
        self.setDword( 24, dword )
        
    def getMessageLength( self ):
        return self.getDword( 24 )
    
    def setFlag( self, dword ):
        self.setDword( 28, dword )
        
    def getFlag( self ):
        return self.getDword( 28 )
    
    def setAcknowledgedIdentifier( self, dword ):
        self.setDword( 32, dword )
        
    def getAcknowledgedIdentifier( self ):
        return self.getDword( 32  )
    
    def setAcknowledgedUniqueID( self, dword ):
        self.setDword( 36, dword )
        
    def getAcknowledgedUniqueID( self ):
        return self.getDword( 36 )
    
    def setAcknowledgedDataSize( self, qword ):
        self.setQword( 40, qword )
        
    def getAcknowledgedDataSize( self ):
        return self.getQword( 40 )
    
    def printNumbers( self ):
        count = 0
        for i in self.data:
            print hex( ord( i ) ),
            
            # :)
            if len( str( hex( ord( i ) ) ) ) == 3:
                print '  ',
            else:
                print ' ',
                
            count += 1
            if count > 15:
                print
                count = 0
                
        print
        
    def printFields( self ):
        print 'SessionID:               ' + str( self.getSessionID() )
        print 'Identifier:              ' + str( self.getIdentifier() )
        print 'Data offset:             ' + str( self.getDataOffset() )
        print 'Total data size:         ' + str( self.getTotalDataSize() )
        print 'Message length:          ' + str( self.getMessageLength() )
        print 'Flag:                    ' + str( self.getFlag() )
        print 'Acknowledged identifier: ' + str( self.getAcknowledgedIdentifier() )
        print 'Acknowledged unique ID:  ' + str( self.getAcknowledgedUniqueID() )
        print 'Acknowledged data size:  ' + str( self.getAcknowledgedDataSize() )
        
    def fill( self, data ):
        self.setSessionID(self.getDwordFromData(data, 0))
        self.setIdentifier(self.getDwordFromData(data, 4))
        self.setDataOffset(self.getQwordFromData(data, 8))
        self.setTotalDataSize(self.getQwordFromData(data, 16))
        self.setMessageLength(self.getDwordFromData(data, 24))
        self.setFlag(self.getDwordFromData(data, 28))
        self.setAcknowledgedIdentifier(self.getDwordFromData(data, 32))
        self.setAcknowledgedUniqueID(self.getDwordFromData(data, 36))
        self.setAcknowledgedDataSize(self.getQwordFromData(data, 40))
    
    def getDwordFromData( self, header, start ):
        '''extract 4 bytes from header starting from start
        and return it as a number'''
    
        b1 = ord( header[ start ] )
        b2 = ord( header[ start + 1 ] )
        b3 = ord( header[ start + 2 ] ) 
        b4 = ord( header[ start + 3 ] )
    
        return (b4<<24) + (b3<<16) + (b2<<8) + b1

    def getQwordFromData( self, header, start ):
        # i think its like this
        return (self.getDwordFromData( header, start + 4) << 32) + self.getDwordFromData( header, start )

