# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from emesenecommon import debugmsg

class SignalHandler:
    '''
    This class is used to connect handlers to signals
    '''
    
    def __init__( self ):
        '''
        contructor, just create the signal dictionary
        '''
        
        self.signals = {}
        
    def connect( self, signal, callback ):
        '''
        connect a signal to a callback, the signal is appended to the
        signal list, the callbacks are called on secuence until one return False
        '''
        
        if self.signals.has_key( signal ):
            self.signals[ signal ].append( callback )
        else:
            self.signals[ signal ] = [ callback ]
          
    def disconnect( self, signal, callback ):
        '''
        disconnect the callback from signal
        '''
        
        if self.signals.has_key( signal ):
            try:
                self.signals[ signal ].pop( self.signals[ signal ].index( callback ) )
            except:
                debugmsg( 'Signal \"' + signal + '\" couldn\'t be disconnected', 3 )
                debugmsg( '\tCallbacks connected: ', 3 )
                for i in self.signals[ signal ]:
                    debugmsg( '\t-' + str( i ), 3)
            
    def emit( self, signal, *args ):
        '''
        emit the signal and call all the callbacks on the signal list until one
        return False
        '''
  
        if self.signals.has_key( signal ):
            for i in self.signals[ signal ]:
                if i( *args ) == False:
                    break