# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import socket
import urllib
import select

import Msnobj
import Socket # class from msnp13
import MsnP2P

def debugmsg( arg, level=5 ):
    if level <=  5:
        print arg

class Switchboard:
    '''
    This class represent a switchboard connection with the notification server.
    '''
    
    def __init__( self , id, msn, type, status = 'pending' ):
        '''class contructor
        type can be 'requested' or 'invited', its 'requested' if we request the
        switchboard and guess what invited means? :P.
        status is pending initialy, then when we process the connection string is
        connected and is stablished when someone join the conversation, 
        if something goes wrong the status is closed'''
        
        self.id = id
        
        self._status = status
        self._validStatus = [ 'pending', 'connected', 'established', 'closed' ]
        self._invalidTransitions = { 'connected' : [ 'pending' ], 'established' : [ 'pending', 'connected' ] }
        self._connectionString = ''
            
        self.msn = msn

        self._user = self.msn._user
        self._debug = self.msn._debug
        self._proxy = self.msn._proxy
        
        self._host = ''
        self._port = 0
        self._authenticationType = ''
        self._authenticationString = ''
        self._sessionID = ''
        self._command = ''
        self._socket = None
        
        self.typingCallback = None
        self.messageCallback = None
        self.nudgeCallback = None
        self.userJoinCallback = None
        self.userLeaveCallback = None
        self.userOfflineCallback = None
        self.userOnlineCallback = None
                
        self.members = {}
        self.messageQueue = [] # a list that holds the messages sent when the status was 'pending
                               # its a list of dicts like this { 'msg' : msg, 'format' : format, 'contentType' : contentType }'
        self._invitationQueue = [] # a list of mails to invite when we go to established status
        
        self.p2p = MsnP2P.P2PManager( self )
        self.msn.connect( 'user offline', self.userOffline )
        self.msn.connect( 'user online', self.userOnline )
        self.msn.connect( 'msnobj changed', self.msnobjChanged )
        
        self.firstMessage = False
        self.firstUser = ''
       
    def __repr__( self ):
        return '<Switchboard users: ' + str( self.members.keys() ) + '>' 
   
    def parse( self ):
        '''parse the connection string to fill the attributes'''
    
        if self._connectionString == '':
            return        

        if self._connectionString.split()[ 0 ] == 'XFR':
            # example
            # XFR 15 SB 207.46.108.37:1863 CKI 17262740.1050826919.32308\r\n
            ( command, tid, sb, host, authenticationType, authenticationString, u, server) = self._connectionString.split()
        else:
            # example
            # RNG 11752013 207.46.108.38:1863 CKI 849102291.520491113 example@passport.com Example%20Name
            ( command, sessionID, host, authenticationType, authenticationString, user, userName, u, server ) = self._connectionString.split()
            self._sessionID = sessionID
             
            self.addMember( user, userName )

        self._command = command
        self._host = host.split( ':' )[ 0 ]
        self._port = int( host.split( ':' )[ 1 ] )
        self._authenticationType = authenticationType
        self._authenticationString = authenticationString
        
    def leaveChat( self ):
        '''leave the conversation'''
        
        self.setStatus( 'closed' )
        try:
            self._socket.sendText( 'OUT\r\n' )
        except Exception, e:
            pass        
        
    def process( self ):
        '''read the info from Socket and process the info'''
        
        if self._socket == None:
            return
        
        fds = self._socket.select()

        if len( fds[ 0 ] ) < 1 or len( fds[ 1 ] ) < 1:
            return
        
        try:
            (command, tid, params) = self._socket.receiveCommand()
        except:
            print str( self ) + ' closed'
            self.setStatus('closed')
            return

        if command == 'MSG':
            # MSG god@gmail.com God%20http://www.400monkeys.com/God/ 111\r\n
            # MIME-Version: 1.0
            # Content-Type: text/plain; charset=UTF-8
            # X-MMS-IM-Format: FN=Sans; EF=; CO=000000; PF=0
            # 
            # a
            
            try:
                message = self._socket.receivePayload( int( params.split()[ 1 ] ) )
            except Exception, e:
                # raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                return
                
            nick = params.split()[ 0 ]
            self._socket.debug( message )
            
            header = message.split( '\r\n\r\n' )[ 0 ]
            body = message.split( '\r\n\r\n' )[ 1 ]
            
            if header.find( 'Content-Type: text/plain;' ) != -1:
                
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new conversation', self.firstUser, self )
                
                format = ''
                if 'X-MMS-IM-Format: ' in header:
                    format = header.split( 'X-MMS-IM-Format: ' )[ 1 ].split( '\r\n' )[ 0 ]
                
                charset = header.split( 'Content-Type: text/plain; charset=' )[ 1 ].split( '\r\n' )[ 0 ]
                
                if self.messageCallback != None:
                    try:
                        self.messageCallback( tid, nick, body, format, charset )    
                    except Exception, e:
                        #raise e
                        self._socket.debug( 'exception running message callback' )
                        self._socket.debug( str( e ) )
                
            elif header.find( 'text/x-msnmsgr-datacast' ) != -1 and body.find( 'ID: 1' ) != -1:
                
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new conversation', self.firstUser, self )
                
                self.msn.emit( 'nudge received', tid )
                if self.nudgeCallback != None:
                    try:
                        self.nudgeCallback( tid )
                    except Exception, e:
                        #raise e
                        self._socket.debug( 'exception running nudge callback' )
                        self._socket.debug( str( e ) )
            elif header.find( 'text/x-msmsgscontrol' ) != -1:
                if self.typingCallback != None:
                    try:
                        self.typingCallback( tid )
                    except Exception, e:
                        #raise e
                        self._socket.debug( 'exception running typing callback' )
                        self._socket.debug( str( e ) )
            elif header.find( 'text/x-mms-emoticon' ) != -1 or\
            header.find( 'text/x-mms-animemoticon' ) != -1:
                self.parseCustomEmoticon( message )
            elif header.find( 'application/x-msnmsgrp2p' ) != -1\
            and header.find( 'P2P-Dest: ' + self.getUser() ) != -1:
                self.p2p.process( message, tid )
                
        elif command == 'USR':
            if params.split( ' ' )[ 0 ] == 'OK':
                self.setStatus( 'connected' )
            else:
                raise 'SwitchboardException'

        elif command == 'IRO':
            #IRO 1 1 1 god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( currentNumber, totalNumber, mail, nick, clientID ) = params.split( ' ' )
            self.addMember( mail, nick )
            
            if self.userJoinCallback != None:
                self.userJoinCallback( mail )
                
            self.setStatus( 'established' ) 
                
        elif command == 'JOI':
            # JOI god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( nick, clientID ) = params.split( ' ' )
            self.addMember( tid, nick )
            
            if self.userJoinCallback != None:
                self.userJoinCallback( tid )
                
            self.setStatus( 'established' ) 
                
        elif command == 'ANS':
            if params == 'OK':
                self.setStatus( 'connected' )                
            else:
                raise 'SwitchboardException'
        elif command == 'BYE':
            self.leave( tid )
            
            # TODO, check when a bye means an OUT
            # from the other side.
            if params == '1':
                if self.userLeaveCallback != None:
                    self.userLeaveCallback( tid )

    def parseCustomEmoticon( self, message ):
        body = message.split( '\r\n\r\n' )[ 1 ]
        l = body.split( '\t' )
        d = {}
      
        while len( l ) > 0:
            if len( l ) < 2:
                break 
            string = l.pop(0)
            msnobjString = l.pop(0)
            msnobj = Msnobj.createFromString( msnobjString, False ) 
            
            if msnobj != None:
                filename = urllib.quote( string )
                completeFileName = self.msn.getCacheDir() + os.sep + filename
                #TODO: erase the or true
                if not self.msn.cacheFileExist( completeFileName ) or True: 
                    self.getCustomEmoticon(msnobj.getCreator(), msnobj, completeFileName)
            

    def sendMessage( self, msg = '',  format = '' , contentType = 'text/plain; charset=UTF-8', acknowledgeType = 'A', extraHeader = '' ):
        if extraHeader != "":
            header = "MIME-Version: 1.0\r\nContent-Type: " + contentType + "\r\n" + extraHeader + '\r\n'
        else:
            header = "MIME-Version: 1.0\r\nContent-Type: " + contentType + "\r\n"
            
        if contentType == 'text/plain; charset=UTF-8':
            msg = msg[ :512 ]
        
        if self._status != 'established':
            self.messageQueue.append( { 'msg' : msg, 'format' : format, 'contentType' : contentType, 'acknowledgeType' : acknowledgeType, 'extraHeader' : extraHeader } )                    

        elif self._status == 'established':
            if format != "":
                header += format + "\r\n\r\n"
            else:
                header += "\r\n"
                
            try:
                self._socket.sendPayloadCommand( 'MSG', acknowledgeType, header + msg )
            except Exception, e:
                #raise e
                print 'socket error on switchboard, closing switchboard'
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )

    def flushMessageQueue( self ):
        '''send all the unsended messages'''
        
        if self._status == 'established':

            # flush the queue
            if len( self.messageQueue ) > 0:
                for i in self.messageQueue:
                    if i['extraHeader'] != "":
                        header = "MIME-Version: 1.0\r\nContent-Type: " + i[ 'contentType' ] + "\r\n" + i['extraHeader'] + '\r\n'
                    else:
                        header = "MIME-Version: 1.0\r\nContent-Type: " + i[ 'contentType' ] + "\r\n"
                    
                    if i[ 'format' ] != "":
                        header += i[ 'format' ] + "\r\n\r\n"
                    else:
                        header += "\r\n"
                
                    try:
                        #if type(i[ 'msg' ]) == list:
                        #    print 'es una lista' + str( i[ 'msg' ] )
                        #    self._socket.sendPayloadCommand( 'MSG', i[ 'acknowledgeType' ], header + i[ 'msg' ][ 0 ] )
                        #else:
                        self._socket.sendPayloadCommand( 'MSG', i[ 'acknowledgeType' ], header + i[ 'msg' ] )
                    except Exception, e:
                        #raise e
                        self._socket.debug( 'socket error on switchboard, closing switchboard' )
                        self._socket.debug( str( e ) )
                        self.setStatus( 'closed' )
                # flushed
                self.messageQueue = []
                self._socket.debug( 'message queue flushed' )

    def sendNudge( self ):
        '''a easy method to send a nudge'''

        self.sendMessage( 'ID: 1\r\n\r\n', '', 'text/x-msnmsgr-datacast' )

    def sendIsTyping( self ):
        '''a easy method to send the is typing message'''

        self.sendMessage( '\r\n', '', 'text/x-msmsgscontrol\r\nTypingUser: ' + self._user )

    def invite( self, mail ):
        '''invite somebody to the switchboard conversation'''

        if self.firstUser == '':
            self.firstUser = mail

        if ( self._status == 'connected' or self._status == 'established' ) and not mail in self.members.keys():
            try:
                self._socket.sendCommand( 'CAL', mail )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                return
            
            if mail in self._invitationQueue:
                self._invitationQueue.pop( self._invitationQueue.index( mail ) )
        elif not mail in self._invitationQueue:
            self._invitationQueue.append( mail )

    def leave( self, mail ):
        '''remove the mail from the members list'''
        
        if mail in self.members.keys():
            if len( self.members ) == 1:
                self.setStatus('closed')
            else:
                del self.members[ mail ]
    
    def setConnectionString( self, connectionString ):
        '''we received the conecction string (the first response of a XFR or the RNG)
        and here we process it'''
        
        self._connectionString = connectionString        
        self.parse()
        self.connect()
        
    def connect( self ):
        '''connect the socket'''
        
        if self._proxy != None:
            self._socket = self.msn.socket #Socket.PSocket( self._host, self._port, self._debug, self._proxy )
        else:
            self._socket = Socket.Socket( self._host, self._port, self._debug )
            self._socket.connect()
        
        if self._command == 'XFR':
            try:
                self._socket.sendCommand( "USR" , self._user + " " + self._authenticationString )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
            
        else: #RNG
            # ANS 1 alice@passport.com 1056411141.26158 17342299\r\n
            try:
                self._socket.sendCommand( "ANS" , self._user + " " + self._authenticationString + " " + self._sessionID )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                        
    def addMember( self, mail, nick ):
        '''add a member to the members dict'''
        
        if self.firstUser == '':
            self.firstUser = mail
        self.members[ mail ] = nick.replace( '%20', ' ' )

    def setStatus( self, status ):
        '''set the status, the status can be 'pending', 'stablished' or 'closed'
        PLEASE change the status here and not directly because we may want to do thing when a status
        is changed'''
        
        if status in self._validStatus and self._status != status:
            if self._invalidTransitions.has_key( self._status ) and status in self._invalidTransitions[ self._status ]:
                return

            self._status = status

            if status == 'connected':
                for i in self._invitationQueue:
                    self.invite( i )
                    
                self._invitationQueue = []
            elif self._status == 'established':
                self.flushMessageQueue()
            
    def getStatus( self ):
        '''return the status'''
        
        return self._status
        
    def setTypingCallback( self, callback ):
        '''this method set the method that will be called when a typing message
        is processed.
        the callback receive 1 parameters, the mail wich is typing'''
        
        self.typingCallback = callback
        
    def setMessageCallback( self, callback ):
        '''this method set the method that will be called when a message
        is processed.
        the callback receive 4 parameters:
         * mail: the mail that send you the message
         * nick: the nick according to the switchboard, maybe is out of date 
         * message: the text of the message
         * format: the format of the message
         * contentType: the contentType'''
        
        self.messageCallback = callback
        
    def setNudgeCallback( self, callback ):
        '''this method set the method that will be called when a nudge message
        is processed.
        the callback receive 1 parameters the mail that send you the nudge'''
        
        self.nudgeCallback = callback
        
    def setUserJoinCallback( self, callback ):
        '''this method set the method that will be called when a user join to the 
        switchboard.
        the callback receive 1 parameter: the mail that has joined'''
        
        self.userJoinCallback = callback
        
    def setUserLeaveCallback( self, callback ):
        '''this method set the method that will be called when the user leaves
        the switchboard.
        the callback receive 1 parameter: the mail that has leaved'''
        
        self.userLeaveCallback = callback

    def setUserOfflineCallback( self, callback ):
        '''this method set the method that will be called when a user in the switchboard gets offline.
        the callback receive 1 parameter: the mail that goes offline'''
        
        self.userOfflineCallback = callback

    def setUserOnlineCallback( self, callback ):
        '''this method set the method that will be called when a user in the switchboard gets offline.
        the callback receive 1 parameter: the mail that goes online'''
        
        self.userOnlineCallback = callback
    
    def getUser( self ):
        '''return the user mail'''
        
        return self._user
    
    def getId( self ):
        '''return the id of the switchboard, this is a unique identification of the switchboars.
        The value is the value of the Trid of the command that created the switchboard or the
        sessionid if the other user started the conversation, the value doesnt mather, what mather is
        that its unique.'''
        
        return self.id
    
    def getOnlineUsers( self ):
        '''This method returns a list ol mails of the contacts who are not offline'''
        
        return self.msn.contactManager.getOnlineUsers()
    
    def getMembers( self ):
        '''return a list of the members in the switchboard'''
        
        return self.members.keys()

    def getInvitedMembers( self ):
        '''return a list of the members invited but not joined'''
        
        return self._invitationQueue
        
    def getUserDisplayName( self, mail ):
        '''return the user display name ot just the mail if it cant be found'''
        
        return self.msn.contactManager.getContactNick( mail )

    def userOffline( self, email ):
        '''Method called when a user get offline'''

        if email in self.members.keys():
            try:
                self.userOfflineCallback( email )
            except Exception, e:
                #raise e
                self._socket.debug( 'exception running user offline callback' )
                self._socket.debug( str( e ) )

            self.leave( email )

    def userOnline( self, email, oldStatus ):
        '''Method called when a user get online'''

        if email in self.members.keys() and oldStatus == 'FLN':
            try:
                self.userOnlineCallback( email )
            except Exception, e:
                #raise e
                self._socket.debug( 'exception running user online callback' )
                self._socket.debug( str( e ) )
                        
    def getNick( self, mail ):
        '''return the nick of the user mail'''
        
        return self.msn.contactManager.getContactNick( mail )
    
    def getDisplayPicture( self, email ):
        '''start a P2P session to get the display picture'''
        
        contact = self.msn.contactManager.getContact( email )
        
        if contact == None:
            print 'contact ' + email + 'not found in getDisplayPicture'
            return
        
        msnobj = contact.getMsnobj()
        
        if msnobj == None:
            print email + ' has no msnobj in getDisplayPicture'
            return
        
        filename = self.msn.getCacheDir() + os.sep + email.split( '@' )[ 0 ] + '.tmp'
        receiver = self.p2p.newReceiver( email, self.getUser(), msnobj, filename )
        message = receiver.inviteMessage()
        self.sendMessage(message, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + email )
            
    def msnobjChanged( self, msnobj ):
        '''callback called when the user change his msnobj'''
        if msnobj.getCreator() == self.firstUser:
            self.getDisplayPicture( msnobj.getCreator() )
            
    def getCustomEmoticon( self, email, msnobj, filename ):
        receiver = self.p2p.newReceiver( email, self.getUser(), msnobj, filename )
        message = receiver.inviteMessage()
        self.sendMessage(message, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + email )
            