# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk
import pango
from emesenecommon import *

class PreferenceWindow( gtk.Window ):
    '''
    This class display a windows where the user
    can modify the behaviour of emesene
    '''
    
    def __init__( self, controller, config, parent ):
        '''
        Contructor
        '''
        
        gtk.Window.__init__( self )
        
        self.controller = controller
        self.set_default_size( 300, 250 )
        self.set_title( _( 'Preferences' ) )
        
        self.config = config
        self.set_transient_for( parent )
        self.set_type_hint( gtk.gdk.WINDOW_TYPE_HINT_DIALOG )
        
        self.set_border_width( 12 )
        self.set_resizable( False )
        vbox = gtk.VBox()
        vbox.set_spacing( 12 )
        hButBox = gtk.HButtonBox()
        hButBox.set_spacing( 6 )
        hButBox.set_layout( gtk.BUTTONBOX_END )
        notebook = gtk.Notebook()
        bClose = gtk.Button( stock=gtk.STOCK_CLOSE )
        
        vbox.pack_start( notebook, False, False)
        hButBox.pack_start( bClose )
        vbox.pack_start( hButBox, False, False )
        
        self.interfaceTab = InterfaceTab( self.controller, self.config )
        self.connectionTab = ConnectionTab( self.config )
        self.colorSchemeTab = ColorSchemeTab(self.controller, self.config)
        
        notebook.append_page( self.interfaceTab, gtk.Label( _( 'Interface' ) ) )
        notebook.append_page( self.connectionTab, gtk.Label( _( 'Connection' ) ) )
        notebook.append_page( self.colorSchemeTab, gtk.Label(_('Color scheme')))
        
        vbox.show_all()
        self.add( vbox )
        
        self.connect( 'delete-event', self.close )
        bClose.connect( 'clicked', self.close )
        
    def close( self, *args ):
        '''
        Close the window
        '''
        
        self.hide()
        self.save()
        self.destroy()
        
    def save( self ):
        '''
        save the actual setting
        '''
        self.interfaceTab.save()
        self.connectionTab.save()
        
class ColorSchemeTab(gtk.VBox):
    '''
    This class represent the tab that contain all the configuration about the colors
    '''
    def __init__(self, controller, config):
        gtk.VBox.__init__(self)
        
        self.config = config
        self.controller = controller
        
        self.typingColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('typingColor')))
        self.typingColor.connect("color-set", self.save)
            
        self.messageWaitingColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('messageWaitingColor')))
        self.messageWaitingColor.connect("color-set", self.save)
            
        self.nudge = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('nudge')))
        self.nudge.connect("color-set", self.save)
            
        self.doNudge = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('doNudge')))
        self.doNudge.connect("color-set", self.save)
        
        self.personalMessageColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('personalMessageColor')))
        self.personalMessageColor.connect("color-set", self.save)
            
        self.contactBlockedColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('contactBlockedColor')))
        self.contactBlockedColor.connect("color-set", self.save)
            
        self.nickColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('nickColor')))
        self.nickColor.connect("color-set", self.save)
        
        self.timestampColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('timestamp')))
        self.timestampColor.connect("color-set", self.save)
        
        self.linkColor = gtk.ColorButton(\
            gtk.gdk.color_parse(self.config.getValue('linkColor')))
        self.linkColor.connect("color-set", self.save)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Contact typing")))
        hbox1 = gtk.HBox()
        hbox1.pack_start(alignment, True, True, 3)
        hbox1.pack_start(self.typingColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Message waiting")))
        hbox2 = gtk.HBox()
        hbox2.pack_start(alignment, True, True, 3)
        hbox2.pack_start(self.messageWaitingColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Nudge")))
        hbox3 = gtk.HBox()
        hbox3.pack_start(alignment, True, True, 3)
        hbox3.pack_start(self.nudge, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Do nudge")))
        hbox4 = gtk.HBox()
        hbox4.pack_start(alignment, True, True, 3)
        hbox4.pack_start(self.doNudge, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Personal msn message")))
        hbox5 = gtk.HBox()
        hbox5.pack_start(alignment, True, True, 3)
        hbox5.pack_start(self.personalMessageColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Contact blocked tag")))
        hbox6 = gtk.HBox()
        hbox6.pack_start(alignment, True, True, 3)
        hbox6.pack_start(self.contactBlockedColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Contact's list")))
        hbox7 = gtk.HBox()
        hbox7.pack_start(alignment, True, True, 3)
        hbox7.pack_start(self.nickColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Timestamp")))
        hbox8 = gtk.HBox()
        hbox8.pack_start(alignment, True, True, 3)
        hbox8.pack_start(self.timestampColor, False, False, 3)
        
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(_("Link on conversation")))
        hbox9 = gtk.HBox()
        hbox9.pack_start(alignment, True, True, 3)
        hbox9.pack_start(self.linkColor, False, False, 3)
        
        vbox = gtk.VBox()
        
        vbox.pack_start(hbox1, False, False, 3)
        vbox.pack_start(hbox2, False, False, 3)
        vbox.pack_start(hbox3, False, False, 3)
        vbox.pack_start(hbox4, False, False, 3)
        vbox.pack_start(hbox5, False, False, 3)
        vbox.pack_start(hbox6, False, False, 3)
        vbox.pack_start(hbox7, False, False, 3)
        vbox.pack_start(hbox8, False, False, 3)
        vbox.pack_start(hbox9, False, False, 3)
        
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scroll.add_with_viewport(vbox)
        
        self.pack_start(scroll, True, True, 0)        
        self.pack_start(gtk.HSeparator(), False, True, 2)
        
        restoreButton = gtk.Button(stock = gtk.STOCK_CLEAR)
        restoreButton.connect("clicked", self.restoreColors)
        
        alignment = gtk.Alignment(0.5, 0.5, 0.5, 0.5)
        alignment.add(restoreButton)
        self.pack_start(alignment, False, False, 3)
        
        self.show_all()
    
    def restoreColors(self, *args):
        self.config.setDefaultColors()
        
        self.typingColor.set_color(\
            gtk.gdk.color_parse(self.config.getValue('typingColor')))
            
        self.messageWaitingColor.set_color(\
            gtk.gdk.color_parse(self.config.getValue('messageWaitingColor')))
            
        self.nudge.set_color(\
            gtk.gdk.color_parse(self.config.getValue('nudge')))
            
        self.doNudge.set_color(\
            gtk.gdk.color_parse(self.config.getValue('doNudge')))
            
        self.personalMessageColor.set_color(\
            gtk.gdk.color_parse(self.config.getValue('personalMessageColor')))
            
        self.contactBlockedColor.set_color(\
            gtk.gdk.color_parse(self.config.getValue('contactBlockedColor')))
            
        self.nickColor.set_color(gtk.gdk.color_parse(self.config.getValue('nickColor')))
        
        self.linkColor.set_color(gtk.gdk.color_parse(self.config.getValue('linkColor')))
        
        self.timestampColor.set_color(gtk.gdk.color_parse(self.config.getValue('timestamp')))
        
        self.controller.themeChanged()
    
    def save(self, *args):
        self.config.setValue('nickColor', rgbToHexa(self.nickColor.get_color()))
        self.config.setValue('contactBlockedColor', \
            rgbToHexa(self.contactBlockedColor.get_color()))
        self.config.setValue('personalMessageColor',\
            rgbToHexa(self.personalMessageColor.get_color()))
        self.config.setValue('nudge', \
            rgbToHexa(self.nudge.get_color()))
        self.config.setValue('doNudge', \
            rgbToHexa(self.doNudge.get_color()))
        self.config.setValue('messageWaitingColor', \
            rgbToHexa(self.messageWaitingColor.get_color()))
        self.config.setValue('typingColor', rgbToHexa(self.typingColor.get_color()))
        self.config.setValue('linkColor', rgbToHexa(self.linkColor.get_color()))
        self.config.setValue('timestamp', rgbToHexa(self.timestampColor.get_color()))
        
        self.controller.themeChanged()
        
class InterfaceTab( gtk.VBox ):
    '''
    This class represent the tab that contain all the configuration about the GUI
    '''
    
    def __init__( self, controller, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )
        
        self.controller = controller
        self.config = config
        
        self.set_spacing( 18 )
        self.set_border_width( 12 )
        
        hbox1 = gtk.HBox()
        hbox1.set_spacing( 12 )
        theme = gtk.Label( _( '_Theme:' ) )
        theme.set_alignment( 0.0, 0.5 )
        theme.set_use_underline( True )
  
        hbox1.pack_start( theme, False, False )

        comboModel = gtk.ListStore( str )
        self.combo = gtk.ComboBox( comboModel )
        cellRenderer = gtk.CellRendererText()
        self.combo.pack_start( cellRenderer, False )
        self.combo.add_attribute( cellRenderer, 'text', 0)
        
        theme.set_mnemonic_widget( self.combo )
        
        done = False
        count = 0
        
        for i in sort( self.config.getThemes() ):
            if not done:
                if i == self.config.getValue( 'theme' ):
                    done = True
                else:
                    count += 1
                
            comboModel.append( [ i ] )
            
        self.combo.set_active( count )
        hbox1.pack_start( self.combo )
        
        self.pack_start( hbox1, False, False )
        
        self.conversationPanel = ConversationPanel( self.controller, self.config )
        
        frame = gtk.Frame( _( 'Conversation' ) )
        frame.add( self.conversationPanel )
        
        self.combo.connect( 'changed', self.themeChanged )
        self.pack_start( frame )
        self.show_all()
        
    def themeChanged( self, combo ):
        self.config.setValue( 'theme', self.combo.get_active_text() )
        theme = self.controller.themeChanged()
        if theme == 'default':
            self.combo.set_active( sort( self.config.getThemes()).index( 'default' ) )
        
    def save( self ):
        '''
        save the actual setting
        '''
        
        self.config.setValue( 'theme', self.combo.get_active_text() )
        self.conversationPanel.save()
        
class ConnectionTab( gtk.VBox ):
    '''
    This class represent the tab that contain all the configuration about the GUI
    '''
    
    def __init__( self, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )

        self.config = config

        self.set_spacing( 18 )
        self.set_border_width( 12 ) 

        self.debug = gtk.CheckButton( _( '_Show debug in console' ) )
        self.debug.set_active( bool( int( self.config.getValue( 'debug' ) ) ) )
        self.useProxy = gtk.CheckButton( _( '_Use proxy' ) + ' ' + _( '(Experimental)' ) )
        self.useProxy.set_active( bool( int( self.config.getValue( 'useProxy' ) ) ) )
        
        self.proxySettings = ProxySettings( self.config )
        # Proxy fields status
        if self.useProxy.get_active():
            self.proxySettings.enable()
        else:
            self.proxySettings.disable()
        
        frame = gtk.Frame( _( 'Proxy settings' ) )
        frame.add( self.proxySettings )
        
        alig = gtk.Alignment( xscale=1 )
        alig.set_padding( 0, 0, 24, 0 )
        alig.add( frame )

        self.pack_start( self.debug, False, False )
        
        proxyBox = gtk.VBox( spacing=12 )
        proxyBox.pack_start( self.useProxy, False, False )
        proxyBox.pack_start( alig, False, False )
        
        self.pack_start( proxyBox, False, False )

        self.useProxy.connect( 'toggled', self.useProxyToggled )

        self.show_all()
    
    def useProxyToggled( self, *args ):
        '''
        callback for the toggled signal
        '''
        
        if self.useProxy.get_active():
            self.proxySettings.enable()
        else:
            self.proxySettings.disable()
    
    def save( self ):
        '''
        save the actual setting
        '''
    
        self.config.setValue( 'debug', str( int( self.debug.get_active() ) ) )
        self.config.setValue( 'useProxy', str( int( self.useProxy.get_active() ) ) )
        self.proxySettings.save()
        
class ConversationPanel( gtk.VBox ):
    '''
    This class represent the panel to configure the conversation inside the interface tab
    '''
        
    def __init__( self, controller, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )      

        self.controller = controller
        self.config = config
        
        self.set_spacing( 6 )
        self.set_border_width( 12 )

        self.bFont = gtk.FontButton( self.config.getValue( 'font' ) + \
                                    ' ' + self.config.getValue( 'fontSize' ) )
        self.bColor = gtk.ColorButton( gtk.gdk.color_parse(\
            self.config.getValue('color')))
        self.logPath = gtk.Entry()      
        self.logPath.set_text( self.config.getValue( 'logPath' ) )
        
        #tabbed chat check button
        self.tabbedChat = gtk.CheckButton( _( 'T_abbed chat:' ) )
        self.tabbedChat.set_active( bool( int( self.config.getValue( 'tabbedChat' ))))
                    
        #groups elements for tabbed chat
        hboxTabbedChat = gtk.HBox()
        hboxTabbedChat.pack_start(self.tabbedChat, True, True, 0)
        
        #tab position combo box
        comboModel = gtk.ListStore( str )
        self.comboBoxTabPosition = gtk.ComboBox( comboModel )
        cellRenderer = gtk.CellRendererText()
        self.comboBoxTabPosition.pack_start( cellRenderer, False )
        self.comboBoxTabPosition.add_attribute( cellRenderer, 'text', 0)
        
        self.comboBoxTabPosition.append_text(_('Top'))
        self.comboBoxTabPosition.append_text(_('Bottom'))
        self.comboBoxTabPosition.append_text(_('Left'))
        self.comboBoxTabPosition.append_text(_('Right'))
        
        hboxTabbedChat.pack_start(self.comboBoxTabPosition, True, True, 0)
        self.pack_start(hboxTabbedChat)
        
        self.tabbedChat.connect("clicked", self.configureComboBoxTabPosition)
        self.configureComboBoxTabPosition()
        
        self.comboBoxTabPosition.connect("changed", self.on_comboBoxPosition_changed)
        
        #show tab if one check button
        self.showTabsIfOne = gtk.CheckButton( _( '_Show tabs if there is only one' ))
        self.showTabsIfOne.set_active( bool( int( \
                            self.config.getValue( 'showTabsIfOne' ) ) ) )
        
        self.pack_start(self.showTabsIfOne, expand=True, fill=True, padding=0)
        
        self.showTimestamp = gtk.CheckButton( _( '_Show time stamp in conversations' ))
        self.showTimestamp.set_active( bool( int( \
                            self.config.getValue( 'showTimestamp' ) ) ) )
        
        self.pack_start(self.showTimestamp, expand=True, fill=True, padding=0)
        
                
        log = gtk.Label( _( '_Log path:' ) )
        log.set_alignment( 0.0, 0.5 )
        log.set_use_underline( True )
        log.set_mnemonic_widget( self.logPath )
        font = gtk.Label( _( '_Default font:' ) )
        font.set_alignment( 0.0, 0.5 )
        font.set_use_underline( True )
        font.set_mnemonic_widget( self.bFont )
        color = gtk.Label( _( 'D_efault color:' ) )
        color.set_alignment( 0.0, 0.5 )
        color.set_use_underline( True )
        
        bColorAlig = gtk.Alignment( 0.0, 0.5 )
        bColorAlig.add( self.bColor )
        color.set_mnemonic_widget( self.bColor )
        
        hboxPath = gtk.HBox()
        hboxPath.pack_start(log, True, True, 0)
        hboxPath.pack_start(self.logPath, True, True, 0)
        
        self.pack_start(hboxPath)
        
        hboxFont = gtk.HBox()
        hboxFont.pack_start(font, True, True, 0)
        hboxFont.pack_start(self.bFont, True, True, 0)
        
        self.pack_start(hboxFont)
        
        hboxColor = gtk.HBox()
        hboxColor.pack_start(color, True, True, 0)
        hboxColor.pack_start(bColorAlig, True, True, 0)
        
        self.pack_start(hboxColor)
        
        self.show_all()
        
    def on_comboBoxPosition_changed(self, *args):
        self.save()
        self.controller.updateTabPositionIfTabbedChat()
    
    def configureComboBoxTabPosition(self, *args):
        
        if not self.tabbedChat.get_active():
            self.comboBoxTabPosition.set_sensitive(False)
        else:
            self.comboBoxTabPosition.set_sensitive(True)
        
        tabPosition = int(self.config.getValue('tabPosition'))
        self.comboBoxTabPosition.set_active(tabPosition)
        
    def getTabPosition(self):
        """Gets the actual tab position according to the active 
            comboBoxTabPosition item
        """
        return self.comboBoxTabPosition.get_active()
        
    def save( self ):
        '''
        save the actual setting
        '''
        color = self.bColor.get_color()
        if self.controller.msn != None:
            self.controller.msn.emit( 'color changed', rgbToHexa(color) )
        
        pangoDesc = pango.FontDescription( self.bFont.get_font_name() )
        font = pangoDesc.get_family()
        style = pangoDesc.get_style()
        weight = pangoDesc.get_weight()

        italic = bold = False

        if style == pango.STYLE_ITALIC:
            italic = True
        if weight == pango.WEIGHT_BOLD:
            bold = True
            
        if self.controller.msn != None:
            self.controller.msn.emit( 'font changed', font, italic, bold, pangoDesc.get_size() )
            
        size = str( pango.FontDescription( \
                self.bFont.get_font_name() ).get_size() / pango.SCALE )
        
        log = self.logPath.get_text()
        tabbed = str( int( self.tabbedChat.get_active() ) )
        position = str(self.getTabPosition())
        showIfOne = str( int( self.showTabsIfOne.get_active() ) )
        showTimestamp = str( int( self.showTimestamp.get_active() ) )
        
        self.config.setValue( 'color', rgbToHexa(color) )
        self.config.setValue( 'font', font )
        self.config.setValue( 'fontSize', size )
        self.config.setValue( 'logPath', log )
        self.config.setValue( 'tabbedChat', tabbed )
        self.config.setValue( 'tabPosition', position)
        self.config.setValue( 'showTabsIfOne', showIfOne )
        self.config.setValue( 'showTimestamp', showTimestamp )
        
class ProxySettings( gtk.VBox ):
    '''
    This class represent the panel with the proxy variables in the config file
    '''

    def __init__( self, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )   

        self.config = config
        
        self.set_spacing( 6 )
        self.set_border_width( 12 )  

        self.host = gtk.Entry()
        self.host.set_text( self.config.getValue( 'proxyHost' ) )
        self.port = gtk.Entry()
        self.port.set_text( self.config.getValue( 'proxyPort' ) )

        table = gtk.Table( 2, 2 )
        table.set_row_spacings( 6 )
        table.set_col_spacings( 12 )
        
        host = gtk.Label( _( '_Host:' ) )
        host.set_alignment( 0.0, 0.5 )
        host.set_use_underline( True )
        host.set_mnemonic_widget( self.host )
        port = gtk.Label( _( '_Port:' ) )
        port.set_alignment( 0.0, 0.5 )
        port.set_use_underline( True )
        port.set_mnemonic_widget( self.port )
        table.attach( host, 0, 1, 0, 1 )
        table.attach( port , 0, 1, 1, 2 )
        
        table.attach( self.host, 1, 2, 0, 1 )
        table.attach( self.port, 1, 2, 1, 2 )
        
        self.pack_start( table )

        self.show_all()
    
    def save( self ):
        '''
        save the actual setting
        '''
        
        self.config.setValue( 'proxyHost', self.host.get_text() )
        self.config.setValue( 'proxyPort', self.port.get_text() )
    
    def enable( self ):
        '''
        enable the entries for editing
        '''
        
        self.host.set_editable( True )
        self.port.set_editable( True )
        self.set_sensitive( True )

    def disable( self ):
        '''
        disable the entries for editing
        '''
        
        self.host.set_editable( False )
        self.port.set_editable( False )
        self.set_sensitive( False )

if __name__ == "__main__":
    PreferenceWindow()
