# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk

from Msnp13 import reverse_status

import UserMenu
import GroupMenu
import TreeViewTooltips

import pango

from msnPlusMarkup import msnPlusMarkup


class UserList( gtk.TreeView ):
    """This class represent the user list in the main window"""

    def __init__( self , controller ):
        '''
        Constructor
        '''

        gtk.TreeView.__init__( self )
        self.scroll = None
        self.controller = controller
        self.config = self.controller.config
        self.contactManager = self.controller.getContactManager()
        self.theme = self.controller.theme
        self.groupState = {}
        self.dictEmailIterator = {}

        self.cellRendererText = gtk.CellRendererText()
        self.cellRendererPixbuf = gtk.CellRendererPixbuf()
        self.cellRendererPixbufSpace = gtk.CellRendererPixbuf()
        
        nickColor = self.config.getValue('nickColor')
        self.cellRendererText.set_property("foreground-gdk",\
            gtk.gdk.color_parse(nickColor))
        self.cellRendererText.set_property( 'xalign', 0.0 )
        self.cellRendererPixbuf.set_property( 'xalign', 0.0 )
        self.cellRendererPixbufSpace.set_property( 'xalign', 0.0 )
        self.cellRendererPixbufSpace.set_property( 'visible', False )
        self.cellRendererPixbuf.set_property( 'visible', False )

        self.columnType = [ gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, str, str, str, bool ]
        column = gtk.TreeViewColumn()
        column.set_title('Icons & Text')
        column.set_expand( True )
        self.append_column( column )

        column.pack_start(self.cellRendererPixbufSpace, False)
        column.pack_start(self.cellRendererPixbuf, False)
        column.pack_start(self.cellRendererText )
        column.add_attribute(self.cellRendererPixbuf, 'pixbuf', 1)
        column.add_attribute(self.cellRendererPixbufSpace, 'pixbuf', 0)
        column.add_attribute(self.cellRendererPixbuf, 'visible', 5)
        column.add_attribute(self.cellRendererPixbufSpace, 'visible', 5)
        #column.add_attribute(self.cellRendererText, 'text', 2)
        column.set_attributes(self.cellRendererText, markup=2)
        self.cellRendererText.set_property( 'ellipsize', pango.ELLIPSIZE_END )

        self.treeStore = gtk.TreeStore( *self.columnType )
        self.set_model( self.treeStore )
        self.set_search_column( 2 )
        self.set_headers_visible( False )
        self.set_reorderable( True )

        self.connect( "row_activated" , self.row_activated )
        self.connect( "button-press-event" , self.button_press_event )
        self.connect( "row-expanded" , self.row_expanded )
        self.connect( "row-collapsed" , self.row_collapsed)

        #self.connect("motion-notify-event", self.mne )
        #self.connect( "leave-notify-event" , self.clearStatus )
        #self.set_events( gtk.gdk.POINTER_MOTION_MASK )

        self.showByNick = bool(int(self.controller.config.getValue("showByNick")))
        self.showOffline = bool(int(self.controller.config.getValue("showOffline")))
        self.showByStatus = bool(int(self.controller.config.getValue("showByStatus")))
        self.showEmptyGroups = bool(int(self.controller.config.getValue("showEmptyGroups")))

        self.tooltips = TreeViewTooltips.TreeViewTooltips(self, info_col=2, mail_col=3, is_user_col=5)
        
        self.plusMarkup = msnPlusMarkup()

        self.show()

    def add( self , values , iterator = None ):
        return self.treeStore.append( iterator, values )

    def setShowByNick( self , value ):
        self.showByNick = value

    def getShowByNick( self ):
        return self.showByNick

    def setShowOffline( self , value ):
        self.showOffline = value

    def getShowOffline( self ):
        return self.showOffline

    def setShowEmptyGroups( self , value ):
        self.showEmptyGroups = value

    def getShowEmptyGroups( self ):
        return self.showEmptyGroups

    def setShowByStatus( self , value ):
        self.showByStatus = value

    def getShowByStatus( self ):
        return self.showByStatus

    def orderByStatus( self, x ):
    	if x in self.controller.status_ordered[ 1 ]:
    		return self.controller.status_ordered[ 1 ].index(x)
    	else:
            # this is only supposed to happen when not showByStatus
            ret = x.lower()
            if ret == 'no group':
                return 'zzzzzzzzz'    # TODO sth better
            else:
                return ret

    def refresh( self ):
        self.treeStore.clear()
        
        nickColor = self.config.getValue('nickColor')
        self.cellRendererText.set_property("foreground-gdk",\
            gtk.gdk.color_parse(nickColor))
        
        self.showByNick = bool(int(self.controller.config.getValue("showByNick")))
        self.showOffline = bool(int(self.controller.config.getValue("showOffline")))
        self.showByStatus = bool(int(self.controller.config.getValue("showByStatus")))
        self.showEmptyGroups = bool(int(self.controller.config.getValue( "showEmptyGroups" )))
        self.groupByStatus = bool(int(self.controller.config.getValue( "orderByStatus" )))

        contactListDict = self.contactManager.\
                        getContactList( self.showOffline, self.showEmptyGroups, self.groupByStatus )

        nodes = contactListDict.keys()
        nodes.sort( key=lambda x: self.orderByStatus( x ) )
        
        for i in nodes:
            iterator = None
            #a dict to count users size per status
            statusDictCount = {}
            iterator = self.addListNode(i, len( contactListDict[ i ] ) )

            for j in self.orderContacts(contactListDict[i]):
                self.addContactToList(j, iterator)
            
            if self.groupByStatus:
                # if the key isnt in the dict we add it to True (expanded) except
                # for offline status
                if not self.groupState.has_key(i):
                    if i == "offline":
                        self.groupState[i] = False
                    else:
                        self.groupState[i] = True
                        self.expand_row(self.treeStore.get_path(iterator), True)
                else:
                    if self.groupState[i]:
                        self.expand_row(self.treeStore.get_path(iterator), True)
            else:
                #if the key isnt in the dict we add it as expanded
                if not self.groupState.has_key(i) or self.groupState[i]:
                    self.groupState[i] = True
                    self.expand_row(self.treeStore.get_path(iterator) , True)

    def addListNode(self, node, size=0 ):
        """this method a note to user list: status if showByStatus is True, group name otherwise"""
        if self.groupByStatus:
            if node == 'offline':
                #: in plural
                groupName = _( 'Offline' )
            else:
                #: in plural
                groupName = _( 'Online' )
                
            iterator = self.add( [ None, None, '<b>' + groupName  \
                                  + ' (' + str( size ) + ')' + '</b>',\
                                  node, "group", False ], None )
                
        else:

            if node == 'No group':
                userRelation = self.contactManager.getOnOffUsersRelationByGroup('No group')
                userRelationStr = " (" +str(userRelation[0])+ "/" + str(userRelation[1])+ ")"

                iterator = self.add( [ None, None, '<b>' + _('No group') + \
                    userRelationStr  + '</b>', node, "no group", False ], None )
            else:
                userRelation = self.contactManager.getOnOffUsersRelationByGroup(node)
                userRelationStr = " (" +str(userRelation[0])+ "/" + str(userRelation[1])+ ")"
                
                iterator = self.add( [ None, None, '<b>' + node + \
                    userRelationStr + '</b>', node, "group", False ], None )
                
        return iterator
    
    def addContactToList(self, contactEmail, iterator):
        """ THis method add a contact to user list, according to node (iterator)"""
        
        space = None
        self.dictEmailIterator[contactEmail] = iterator
        
        if self.contactManager.getContactHasSpace(contactEmail):
            space = self.theme.getImage('space')
        
        if not self.contactManager.getContact(contactEmail).getReverse():
            space = self.theme.getImage('reverse')

        if self.contactManager.getContactStatus(contactEmail) == 'FLN' and\
                self.contactManager.getContactHasMobile(contactEmail):
            status = self.theme.getImage('mobile')
        else:
            status = self.theme.statusToPixbuf(\
                self.contactManager.getContactStatus(contactEmail) )

        displayName = self.contactManager.getContactNameToDisplay( contactEmail )

        personalMessageColor = self.config.getValue('personalMessageColor')
        personalMessage = self.contactManager.\
                        getContactPersonalMessage(contactEmail, True)

        if personalMessage == '':
            personalMessage = reverse_status[ self.controller.msn.\
                contactManager.getContactStatus(contactEmail)]

        if self.contactManager.getContactIsAllowed(contactEmail) or\
                self.contactManager.getContactIsBlocked(contactEmail):

            if self.contactManager.getContactIsBlocked(contactEmail):
                status = self.theme.getBlockedImage( status )

            if self.showByNick:
                self.add([space, status,
                            self.plusMarkup.replaceMarkup(displayName) + ' - ' +
                            '<span foreground="'+personalMessageColor+'">' +
                            personalMessage + '</span>' , contactEmail,
                            "user", True ], iterator )
            else:
                self.add([space, status, contactEmail + ' - ' +
                            '<span foreground="'+personalMessageColor+'">' +
                            personalMessage + '</span>' , contactEmail,
                            "user", True], iterator)
    
    def mne( self , widget , e ):
        _list = widget.get_path_at_pos( int(e.x), int(e.y) )

        if _list != None:
            iterator = widget.get_model().get_iter( _list[ 0 ] )
            value = widget.get_model().get_value( iterator , 3 )

            self.controller.showUserData( value )

    def clearStatus( self , *args ):
        self.controller.showUserData( "" )

    def row_activated( self , treeview , path , view_column  ):
        iterator = treeview.get_model().get_iter( path )

        mail = treeview.get_model().get_value( iterator , 3 )

        typeOfRow = treeview.get_model().get_value( iterator , 4 )

        if typeOfRow == "user" and \
                not self.contactManager.getContactStatus( mail ) == 'FLN':
            self.controller.newConversation( mail )

    def button_press_event( self , treeview , event ):
        
        self.tooltips.hideTooltip()
                
        if event.button == 3:

            _list = treeview.get_path_at_pos( int(event.x), int(event.y) )

            if _list == None:
                return

            iterator = treeview.get_model().get_iter( _list[ 0 ] )
            rowType = treeview.get_model().get_value( iterator , 4 )
            time = event.time

            if rowType == 'group':
                if not bool(int(self.controller.config.getValue( "orderByStatus" ))):
                    value = treeview.get_model().get_value( iterator , 3 )
                    value = unicode( value )
                    menu = GroupMenu.GroupMenu( self.controller , value )
                    menu.popup( None, None, None, event.button, time )
            elif rowType == 'user':
                value = treeview.get_model().get_value( iterator , 3 )

                groupIter = treeview.get_model().iter_parent( iterator )
                groupName = treeview.get_model().get_value( groupIter , 3 )
                groupName = unicode( groupName )
                
                menu = UserMenu.UserMenu( self.controller , value, groupName )
                menu.popup( None, None, None, event.button, time )

    def row_expanded(self,treeview, iterator, path):
        """
        Callback for a node (usergroup) when expanded, it adds the node
        path to a list of expanded nodes if this one is not allready.
        """

        value = treeview.get_model().get_value( iterator , 3 )
        self.groupState[ value ] = True

    def row_collapsed(self,treeview, iterator, path):
        """
        Callback for a node (usergroup) when collapsed, it removes
        a node path from a list of expanded nodes if present.
        """

        value = treeview.get_model().get_value( iterator , 3 )
        self.groupState[ value ] = False
        
    def getMenuData( self ):
        '''
        Returns useful data for building menus in the format
        ( <selected item type>, [ [ <item name> ], <item's group> ] )
        '''
        try:
            model = self.treeStore
            iterator = self.get_selection().get_selected()[ 1 ]
            l = model.get( iterator , 4 )
            rowType = l[0]

            if rowType == 'group':
                if not bool(int(self.controller.config.getValue( "orderByStatus" ))):
                    value = unicode( model.get_value( iterator , 3 ) )
                    return ( 'group', value )
                else:
                    return ( '', )
            elif rowType == 'user':
                value = model.get_value( iterator , 3 )
    
                groupIter = model.iter_parent( iterator )
                groupName = unicode( model.get_value( groupIter , 3 ) )
    
                return ( 'user', value, groupName )
            else:
                return ( '', )
        except:
            return ( '', )

    def getUserSelected( self ):
        try:
            l = self.treeStore.get( self.get_selection().get_selected()[ 1 ] ,\
                                     3, 4 )
            if l[ 1 ] == "user":
                retval = l[ 0 ]
            else:
                retval = ""
        except:
            retval = ""

        return retval

    def getGroupSelected( self ):
        try:
            l = self.treeStore.get( self.get_selection().get_selected()[ 1 ] ,\
                 2 , 3 , 4 )
            if l[ 2 ] == "group":
                retval = l[ 1 ]
            else:
                retval = ""
        except:
            retval = ""

        return retval

    def getGroupState( self ):
        return self.groupState

    def setGroupState( self , groupState ):
        self.groupState = groupState

    def orderContacts( self, contacts ):
        '''
        receive a dict of contacts and order acording to thei status
        return a list of keys
        '''
        contactByStatus = {}
        returnList = []
        
        if self.showByStatus:
            statusOrdered = self.controller.status_ordered[0]
        else:
            statusOrdered = [ 'noStatus', 'FLN' ]
        
        for i in statusOrdered:
            contactByStatus[i] = []
        
        for i in contacts.iterkeys():
            contactStatus = self.contactManager.getContactStatus(i)
            if not self.showByStatus and not contactStatus == 'FLN':
                contactStatus = 'noStatus'
            contactByStatus[contactStatus].append(i)

        for i in statusOrdered:
            if self.showByNick and len(contactByStatus[i]) > 0:
                # order contacts alphabetically (case ins) by their display name
                contactByStatus[i].sort( key=lambda x: str.lower(\
                            str( self.contactManager.getContactNick ( x ) ) ) )
            else:
                contactByStatus[i].sort()

            returnList += contactByStatus[i]

        return returnList
