# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import gobject
from Msnp13 import msnp13

class UserPanel( gtk.HBox ):
    "this class is the panel that will contain the user photo , his nick and his status"
    
    def __init__( self , controller ):
        "Constructor"
        gtk.HBox.__init__( self )
        
        self.set_border_width( 4 )
        self.set_spacing( 4 )
        
        self.controller = controller
        self.nick = self.controller.msn.nick
        
        self.tNick = gtk.Entry()
        self.tNick.set_text( self.nick )
        
        self.tPersonalMessage = gtk.Entry()
        
        self.image = gtk.Image()
        if self.controller.avatar != None:
            self.image.set_from_pixbuf(self.controller.avatar.getThumb())
        else:
            self.pixbuf = self.controller.theme.getImage( 'userPanel' )
            self.image.set_from_pixbuf( self.pixbuf )
        
        self.tNick.connect( 'activate' , self.on_nick_changed )
        self.tNick.connect( 'focus-out-event' , self.on_nick_changed )
        self.tPersonalMessage.connect( 'activate' , self.on_pm_changed )
        self.tPersonalMessage.connect( 'focus-out-event' , self.on_pm_changed )
        self.tPersonalMessage.set_text( self.controller.msn.personalMessage )
        self.controller.msn.connect( 'self personal message changed', self.personalMessageChanged )
        self.controller.msn.connect( 'self nick changed', self.selfNickChanged )
        self.vbox = gtk.VBox()
        
        self.pack_start( self.image, False, False )
        
        vbox1 = gtk.VBox()
        nick = gtk.Label( _( "Nick:" ) )
        nick.set_alignment( 0.0, 0.5 )
        nick.set_padding( 8, 0 )
        message = gtk.Label( _( "Message:" ) )
        message.set_alignment( 0.0, 0.5 )
        message.set_padding( 8, 0 )
        vbox1.pack_start( nick , True , True )
        vbox1.pack_start( message , True , True )
        
        vbox2 = gtk.VBox()
        vbox2.pack_start( self.tNick , False , False )
        vbox2.pack_start( self.tPersonalMessage , False , False )
        
        hbox = gtk.HBox()
        hbox.pack_start( vbox1, False, False )
        hbox.pack_start( vbox2, True, True )
                
        self.vbox.pack_start( hbox , False , False )
        self.pack_start( self.vbox )
        
        self.show_all()
            
    def selfNickChanged( self, nick ):
        '''method called when the user change his nick in other part'''
        
        self.tNick.set_text( nick )
        
    def on_nick_activate( self , *args ):
        self.controller.changeNick( self.tNick.get_text() )
        
    def on_nick_changed( self, *args ):
        if not self.controller.connected:
            return
        if self.controller.msn.nick != self.tNick.get_text():
            self.controller.changeNick( self.tNick.get_text() )
            
    def on_pm_changed( self, *args ) :
        if not self.controller.connected:
    		return
        if self.controller.msn.personalMessage != self.tPersonalMessage.get_text():
            self.controller.changePersonalMessage( self.tPersonalMessage.get_text() )
            
    def personalMessageChanged( self, user, pm ):
        '''method called when the pm is changed in other place'''
        
        self.tPersonalMessage.set_text( pm )
            
    def on_personal_activate( self , *args ):
        self.controller.changePersonalMessage( self.tPersonalMessage.get_text() )
        
    def nickChanged( self , nick ):
        self.tNick.set_text( nick )
            
    def setAvatar( self, pixbuf ):
        if pixbuf:
            self.image.set_from_pixbuf( pixbuf )
