# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
from operator import truediv
from BeautifulSoup import BeautifulSoup

colorCodes = {}
colorCodes[ '0' ] = 'ffffff'
colorCodes[ '1' ] = '000000'
colorCodes[ '2' ] = '00007D'
colorCodes[ '3' ] = '009100'
colorCodes[ '4' ] = 'FF0000'
colorCodes[ '5' ] = '7D0000'
colorCodes[ '6' ] = '9A009A'
colorCodes[ '7' ] = 'FC7D00'
colorCodes[ '8' ] = 'FFFF00'
colorCodes[ '9' ] = '00FC00'
colorCodes[ '10' ] = '009191'
colorCodes[ '11' ] = '00FFFF'
colorCodes[ '12' ] = '1E1EFC'
colorCodes[ '13' ] = 'FF00FF'
colorCodes[ '14' ] = '7D7D7D'
colorCodes[ '15' ] = 'D2D2D2'
colorCodes[ '16' ] = 'E7E6E4'
colorCodes[ '17' ] = 'cfcdd0'
colorCodes[ '18' ] = 'ffdea4'
colorCodes[ '19' ] = 'ffaeb9'
colorCodes[ '20' ] = 'ffa8ff'
colorCodes[ '21' ] = 'c1b4fc'
colorCodes[ '22' ] = 'bafbe5'
colorCodes[ '23' ] = 'ccffa3'
colorCodes[ '24' ] = 'fafda2'
colorCodes[ '25' ] = 'b6b4b7'
colorCodes[ '26' ] = 'a2a0a1'
colorCodes[ '27' ] = 'f9c152'
colorCodes[ '28' ] = 'ff6d66'
colorCodes[ '29' ] = 'ff62ff'
colorCodes[ '30' ] = '6c6cff'
colorCodes[ '31' ] = '68ffc3'
colorCodes[ '32' ] = '000000'
colorCodes[ '33' ] = 'f9ff57'
colorCodes[ '34' ] = '858482'
colorCodes[ '35' ] = '6e6d7b'
colorCodes[ '36' ] = 'ffa01e'
colorCodes[ '37' ] = 'F92411'
colorCodes[ '38' ] = 'FF1EFF'
colorCodes[ '39' ] = '1E29FF'
colorCodes[ '40' ] = '7dffa5'
colorCodes[ '41' ] = '60f913'
colorCodes[ '42' ] = 'fff813'
colorCodes[ '43' ] = '5e6464'
colorCodes[ '44' ] = '4b494c'
colorCodes[ '45' ] = 'd98812'
colorCodes[ '46' ] = 'eb0505'
colorCodes[ '47' ] = 'de00de'
colorCodes[ '48' ] = '0000d3'
colorCodes[ '49' ] = '03cc88'
colorCodes[ '50' ] = '59d80d'
colorCodes[ '51' ] = 'd4c804'
colorCodes[ '52' ] = '333335'
colorCodes[ '53' ] = '18171c'
colorCodes[ '54' ] = '944e00'
colorCodes[ '55' ] = '9b0008'
colorCodes[ '56' ] = '980299'
colorCodes[ '57' ] = '01038c'
colorCodes[ '58' ] = '01885f'
colorCodes[ '59' ] = '389600'
colorCodes[ '60' ] = '9a9e15'
colorCodes[ '61' ] = '473400'
colorCodes[ '62' ] = '4d0000'
colorCodes[ '63' ] = '5f0162'
colorCodes[ '64' ] = '000047'
colorCodes[ '65' ] = '06502f'
colorCodes[ '66' ] = '1c5300'
colorCodes[ '67' ] = '544d05'

#Thanks Dx for all the Regex help
colorCodesHex = re.compile('\[[cC]=#([0-9A-Fa-f]{1,6})\](.*?)\[/[cC]\]')
colorCodesCode = re.compile('\[[cC]=([0-9]{1,2})\](.*?)\[/[cC]\]')
gradientColorsCode = re.compile('\[[cC]=([0-9]{1,2})\](.*?)\[/[cC]=([0-9]{1,2})\]')
gradientBackColorsCode = re.compile('\[[aA]=([0-9]{1,2})\](.*?)\[/[aA]=([0-9]{1,2})\]')


backColorCodesHex = re.compile('\[[aA]=#([0-9A-Fa-f]{1,6})\](.*?)\[/[aA]\]')
backColorCodesCode = re.compile('\[[aA]=([0-9]{1,2})\](.*?)\[/[aA]\]')

#colorIrcCode = re.compile("·\$([0-9]{1,2}),([0-9]{1,2})|·\$([0-9]{1,2}).*?")
colorIrcCode = re.compile("·\$([0-9]{1,2})?,?([0-9]{1,2})?")
#colorIrcRGB = re.compile("·\$\(([0-9]{3}),([0-9]{3}),([0-9]{3})\)")

boldBb = re.compile('(\[[bB]\](.*?)\[/[bB]\])')
boldIrc = re.compile("·\#(.*?)(·\#|$)")
        
underBb = re.compile('(\[[uU]\](.*?)\[/[uU]\])')
underIrc = re.compile("·\@(.*?)(·\@|$)")

italicBb = re.compile('(\[[iI]\](.*?)\[/[iI]\])')
italicIrc = re.compile("·\&(.*?)(·\&|$)")

strikeBb = re.compile('(\[[sS]\](.*?)\[/[sS]\])')
strikeIrc = re.compile("·\'(.*?)(·\'|$)")

pangoToHtml = {
    'b':'font-weight: bold',
    'u':'text-decoration:underline',
    'i':'font-style: italic',
    's':'text-decoration: line-through',
    'foreground':'color',
    'background':'background-color'
    }

class msnPlusMarkup:
    def replaceMarkup( self, text, isHtml=False ):
        '''replace the [b][/b] etc markup for pango  and html markup'''

        self.isHtml = isHtml
        self.more = 0
        self.backgroundColor = self.frontColor = ''
                
        self.openSpan = None
        
        text = text.replace('·&amp;','·&')
        
        self.currentTag = 'b'
        text = re.sub( boldBb, self.bbToTag, text )
        text = re.sub( boldIrc, self.ircToTag, text )

        self.currentTag = 'i'
        text = re.sub( italicBb, self.bbToTag, text )
        text = re.sub( italicIrc, self.ircToTag, text )
        
        self.currentTag = 'u'
        text = re.sub( underBb, self.bbToTag, text )
        text = re.sub( underIrc, self.ircToTag, text )

        self.currentTag = 's'
        text = re.sub( strikeBb, self.bbToTag, text )
        text = re.sub( strikeIrc, self.ircToTag, text )

        self.currentTag=None
        
        text = re.sub( colorCodesHex, self.hexToTag, text )
        text = re.sub( colorCodesCode, self.codeToTag, text )
        
        #self.currentTag='foreground'
        #text = re.sub( gradientColorsCode, self.gradientToTag, text ) ### Kill Cpu :(
        #self.currentTag='background'
        #text = re.sub( gradientBackColorsCode, self.gradientToTag, text  ) ### Kill Cpu :(
        
        
        self.currentTag=None
        text = re.sub( backColorCodesHex, self.bHexToTag, text )
        text = re.sub( backColorCodesCode, self.bCodeToTag, text )
        
        text = re.sub( colorIrcCode, self.ircCodeToTag, text ) 
        
        if self.openSpan != None:
            text += '</span>'


        
        # dont ask me why pango doesnt parse a span when is the beggining
        # of the text
        if text.startswith( '<span' ):
            text = ' ' + text

        soup = BeautifulSoup(text)
        text = soup.__str__()
        return text

    def ircToTag( self, data ):
        if self.isHtml:
            return '<span style="'+pangoToHtml[self.currentTag]+'">'+data.groups()[0]+'</span>'
        else:
            return '<'+self.currentTag+'>'+data.groups()[0]+'</'+self.currentTag+'>'
        
    def bbToTag( self, data ):

        if self.isHtml:
            return '<span style="'+pangoToHtml[self.currentTag]+'">'+data.groups()[1]+'</span>'
        else:
            return '<'+self.currentTag+'>'+data.groups()[1]+'</'+self.currentTag+'>'
        

    def ircCodeToTag( self, data ):
        text = None
        color1 , color2 = data.groups()

        if( self.frontColor != '' and  color1  == None ):
            color1 = self.frontColor

        if( self.backgroundColor != '' and  color2  == None ):
            color2 = self.backgroundColor
        
        self.frontColor = color1
        self.backgroundColor = color2

        if color1 !=  None and color2 != None:
            if colorCodes.has_key(color1) and colorCodes.has_key(color2):
                if self.isHtml:
                    text = '<span style="color:#'+colorCodes[color1].lower()+'; background-color:#'+colorCodes[color2].lower()+'">' 
                else:
                    text = '<span foreground="#'+colorCodes[color1].lower()+'" background="#'+colorCodes[color2].lower()+'">'
            

        elif color1 != None:
            if colorCodes.has_key(color1):
                if self.isHtml:
                    text = '<span style="color:#'+colorCodes[color1].lower()+'">' 
                else:
                    text = '<span foreground="#'+colorCodes[color1].lower()+'">'
                    
        elif color2 != None:
            if colorCodes.has_key(color2):
                if self.isHtml:
                    text = '<span style="background-color:#'+colorCodes[color2].lower()+'">' 
                else:
                    text = '<span background="#'+colorCodes[color2].lower()+'">'
                    
        if self.openSpan != None:
            text = '</span>'+text

        if text != None:
            self.openSpan = True
            return text

        
        
    def hexToTag( self, data ):
        color, text = data.groups()
        if self.isHtml:
            return '<span style="color: #'+ color.lower() +'">'+text+'</span>'
        else:
            return '<span foreground="#'+ color.lower() +'">'+text+'</span>'
            
        
    def codeToTag( self, data ):
        code, text = data.groups()
        if colorCodes.has_key(code):
            if self.isHtml:
                return '<span style="color: #'+ colorCodes[code].lower() +'">'+text+'</span>'
            else:
                return '<span foreground="#'+ colorCodes[code].lower() +'">'+text+'</span>'
        else:
            return text
    
    
            
            
    def bHexToTag( self, data ):
        color, text = data.groups()
        if self.isHtml:
            return '<span style="color: #'+ color.lower() +'">'+text+'</span>'
        else:
            return '<span foreground="#'+ color.lower() +'">'+text+'</span>'
        
    def bCodeToTag( self, data ):
        code, text = data.groups()
        if colorCodes.has_key(code):
            if self.isHtml:
                return '<span style="background-color: #'+ colorCodes[code].lower() +'">'+text+'</span>'
            else:
                return '<span background="#'+ colorCodes[code].lower() +'">'+text+'</span>'
        else:
            return text
            
            
            
    def RGBToHTMLColor( self, rgb_tuple ):
        """ convert an (R, G, B) tuple to #RRGGBB """
        hexcolor = '#%02x%02x%02x' % rgb_tuple
        # that's it! '%02x' means zero-padded, 2-digit hex values
        return hexcolor

    def HTMLColorToRGB( self, colorstring ):
        """ convert #RRGGBB to an (R, G, B) tuple """
        colorstring = colorstring.strip()
        if colorstring[0] == '#': colorstring = colorstring[1:]
        if len(colorstring) != 6:
            raise ValueError, "input #%s is not in #RRGGBB format" % colorstring
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)

    def gradientToTag( self, data ):
        color, text, toColor = data.groups()
        steaps = text.count("")-1

        ret = ''

        r1, g1, b1 = self.HTMLColorToRGB( colorCodes[color] )
        r2, g2, b2 = self.HTMLColorToRGB( colorCodes[toColor] )
    
        rSteap =  truediv( (r1-r2), (steaps-1) )
        bSteap =  truediv( (b1-b2), (steaps-1) )
        gSteap =  truediv( (g1-g2), (steaps-1) )

        for i in xrange(0,steaps):
            #print (int(r1-(i*rSteap)), int(b1-(i*bSteap)), int(g1-(i*gSteap)))
            if self.isHtml:
                ret += '<span style="'+pangoToHtml[self.currentTag]+': '+ self.RGBToHTMLColor( ( int(r1-(i*rSteap)), int(g1-(i*gSteap)),int(b1-(i*bSteap)) ) ) +'">'+text[i:(i+1)]+'</span>'
            else:
                ret += '<span '+self.currentTag+'="'+ self.RGBToHTMLColor( ( int(r1-(i*rSteap)), int(g1-(i*gSteap)),int(b1-(i*bSteap)) ) ) +'">'+text[i:(i+1)]+'</span>'        
            
        return ret