# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import Plugin

ERROR = ''
try:
    import pynotify
    if not pynotify.init("emesene"):
        ERROR = _( 'there was a problem initializing the pynotify module' )
except:
    ERROR = _( 'there was a problem initializing the pynotify module' )
    
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'Notify about diferent events using pynotify' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _( 'LibNotify' )
        self.name = 'LibNotify'
        
        self.tray = self.controller.trayIcon.getNotifyObject()
        self.pixbuf = self.controller.theme.getImage( 'userPanel' )
        self.notifications = []
        
    def start( self ):
        '''start the plugin'''
        
        self.connect( 'user online', self.online )
        self.enabled = True
        
    def stop( self ):    
        '''
        stop the plugin
        '''
        
        # closes remaining notifications
        for i in self.notifications:
            i.close()

        self.notifications = []
            
        self.disconnect( 'user online', self.online )
        self.enabled = False
        
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        global ERROR
        
        if ERROR != '':
            return ( False, ERROR )
        
        return ( True, 'Ok' )
    
    def online( self, email, oldStatus ):
        '''called when someone get online'''

        if oldStatus != 'FLN':
            return
        
        text = self.controller.msn.contactManager.getContactNameToDisplay( email )
        text += ' - <span foreground="#AAAAAA">(' + email + ')</span>'
        notification = pynotify.Notification("emesene", \
                              text  + "\n" + _( "is now online" ), \
                              attach=self.tray )
        notification.set_icon_from_pixbuf( self.pixbuf )
#        notification.add_action( 'default', 'Open conversation', \
#                                 self.on_notify_action )
        notification.show()
        notification.connect( 'closed', self.on_notify_close )
        self.notifications.append( notification )
        
    def on_notify_close( self, n ):
        if n in self.notifications:
            self.notifications.remove( n )
        
#    def on_notify_action( self, notification, action ):
#        print ref
#        print str[:]
#        self.controller.newConversation( '' )
        