# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import os
import gobject

try:
    import dbus
    DBUS = True
except:
    DBUS = False

import Plugin

from emesenecommon import *

import gettext

gettext.bindtextdomain('plugin_rhythmbox', LANG_PATH)
gettext.textdomain('plugin_rhythmbox')
_ = gettext.gettext
    
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'Show your contacts the song rhythmbox is currently playing. It uses D-Bus' )
        self.authors = { 'Alberto Talavera' : 'talaveran at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = 'Rhythmbox'
        self.name = 'Rhythmbox'
        
        self.bus = dbus.SessionBus()  
        self.loop = None
        self.rbshell = None
        
    def start( self ):
        '''start the plugin'''
              
        self.bus.add_signal_receiver( self.on_playing_uri_changed,
                        'playingUriChanged',
                        'org.gnome.Rhythmbox.Player',
                        None,
                        '/org/gnome/Rhythmbox/Player' )
        self.bus.add_signal_receiver( self.on_playing_changed,
                                     'playingChanged',
                                     'org.gnome.Rhythmbox.Player',
                                     None,
                                     '/org/gnome/Rhythmbox/Player' )
            
        self.enabled = True
    
# need to figure out howto query if a service is availabe
# (we don't want to launch RB!!)
#        # Tries to get current song
#        try:
#            rbplayerobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Player')
#            rbplayer = dbus.Interface(rbplayerobj, 'org.gnome.Rhythmbox.Player')
#            uri = rbplayer.getPlayingUri()
#            self.on_playing_uri_changed( uri )
#        except Exception, e:
#            # RB is not running
#            debugmsg( e )
#            pass
        
#        self.loop = gobject.MainLoop()
#        self.loop.run()
        
    def stop( self ):    
        '''stop the plugin'''
        
        if self.loop:
            self.loop.quit()
        self.bus.remove_signal_receiver( self.on_playing_uri_changed,
                                     'playingUriChanged',
                                     'org.gnome.Rhythmbox.Player',
                                     None,
                                     '/org/gnome/Rhythmbox/Player' )
        self.bus.remove_signal_receiver( self.on_playing_changed,
                                     'playingChanged',
                                     'org.gnome.Rhythmbox.Player',
                                     None,
                                     '/org/gnome/Rhythmbox/Player' )
        self.rbshell = None
        
        self.msn.changeCurrentMedia( "" )
        self.enabled = False
        
    def action( self ):
        pass
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True
        
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if os.name != 'posix':
            return ( False, _( 'This plugin only works in posix systems' ) )
        
        if not DBUS:
            return ( False, _( 'D-Bus cannot be initialized' ) )
        
        return ( True, 'Ok' )
    
    def on_playing_uri_changed( self, uri ):
        
        if not self.enabled:
            return
        
        if not self.rbshell:
            rbshellobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
            rbshell = dbus.Interface(rbshellobj, 'org.gnome.Rhythmbox.Shell')
            self.rbshell = rbshell
            
        self.rbshell.getSongProperties( uri, 
                                   reply_handler=self.changeMediaData, 
                                   error_handler=self.dbusError )
        
    def on_playing_changed( self, playing ):
        if not playing:
            self.msn.changeCurrentMedia( '' )
                
    def changeMediaData( self, data ):
        media = "\\0Music\\01\\0{0} - {1} ({2})\\0%s\\0%s\\0%s\\0\\0" % \
                    ( data['title'], data['artist'], data['album'] )

        self.msn.changeCurrentMedia( media )
                
    def dbusError( self ):
        errormsg( 'Rhythmbox plugin:' )
        errormsg( '\tD-Bus error while fetching current song data' )
            