# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys

if os.name == 'nt':
    import winsound
    
import Plugin

from emesenecommon import *

import gettext

gettext.bindtextdomain('plugin_sound', LANG_PATH)
gettext.textdomain('plugin_sound')
_ = gettext.gettext
    
class Sound:

    def __init__( self , theme ):
    
        self.theme = theme
        self.command = ''
        self.canPlay = False
        
        if os.name == "posix":
            self.checkAvailability()
        else:
            self.canPlay = True
                
    def checkAvailability( self ):
        if self.is_on_path( 'play' ):
            self.canPlay = True
            self.command = 'play'
        elif self.is_on_path( 'aplay' ):
            self.canPlay = True
            self.command = 'aplay'
        
    def play( self , sound ):
        if os.name == "nt":
            winsound.PlaySound( self.theme.getSound( sound ) , winsound.SND_FILENAME | winsound.SND_ASYNC )
        elif os.name == "posix":
            if self.canPlay and self.theme.getSound( sound ):
                os.popen4(self.command + " " + self.theme.getSound( sound ))
            
    def getCommand( self ):
        return self.command
        
    def setCommand( self , string ):
        self.command = string
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        self.theme = controller.theme
        
        self.description = _( 'Play sounds for common events.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _( 'Sound' )
        self.name = 'Sound'
        
        self.sound = Sound(self.theme)
        
        self.config = Plugin.Config( 'Sound' )
        self.playOnline = (self.config.getValue( 'playOnline', '1' ) == '1' )
        self.playMessage = (self.config.getValue( 'playMessage', '1' ) == '1' )
        self.playNudge = (self.config.getValue( 'playNudge', '1' ) == '1' )
        
    def start( self ):
        '''start the plugin'''

        self.enabled = True
        self.msn.connect( 'user online', self.online )
        self.msn.connect( 'message received', self.message )
        self.msn.connect( 'nudge received', self.nudge )
        
    def stop( self ):    
        '''stop the plugin'''
        
        self.enabled = False
        
    def action( self ):
        pass        
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if not self.sound.canPlay:
            return ( False, _( 'play and aplay not found.' ) )
        
        return ( True, 'Ok' )

    def online( self, email, oldStatus ):
        if oldStatus == 'FLN'and self.enabled and self.playOnline:
            self.sound.play( 'online' )

    def message( self ):
        if self.enabled and self.playMessage:
            self.sound.play( 'type' )

    def nudge( self, *args ):
        if self.enabled and self.playNudge:
            self.sound.play( 'nudge' )
            
    def configure( self ):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        l = [ Plugin.Option( 'playOnline', bool, _( 'Play online sound' ), _( 'Play a sound when someone get online' ), (self.config.getValue( 'playOnline', '1' ) == '1') ) ]
        l.append(Plugin.Option( 'playMessage', bool, _( 'Play message sound' ), _( 'Play a sound when someone send you a message' ), (self.config.getValue( 'playMessage', '1' ) == '1') ))
        l.append(Plugin.Option( 'playNudge', bool, _( 'Play nudge sound' ), _( 'Play a sound when someone snd you a nudge' ), (self.config.getValue( 'playNudge', '1' ) == '1') ))
        
        response = Plugin.ConfigWindow( _( 'Config Sound Plugin' ), l ).run()
        if response != None:
            if response.has_key( 'playOnline' ):
                self.config.setValue( 'playOnline', str( int( response[ 'playOnline' ].value ) ) )
            if response.has_key( 'playMessage' ):
                self.config.setValue( 'playMessage', str( int( response[ 'playMessage' ].value ) ) )
            if response.has_key( 'playNudge' ):
                self.config.setValue( 'playNudge', str( int( response[ 'playNudge' ].value ) ) )

        self.playOnline = (self.config.getValue( 'playOnline', '1' ) == '1' )
        self.playMessage = (self.config.getValue( 'playMessage', '1' ) == '1' )
        self.playNudge = (self.config.getValue( 'playNudge', '1' ) == '1' )
        
        self.config.writeConfig()