# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.2'

import gtk
import Plugin
import gettext
import commands
from emesenecommon import *

gettext.bindtextdomain('plugin_spell', LANG_PATH)
gettext.textdomain('plugin_spell')
_ = gettext.gettext

ERROR = ''

try:
    import gtkspell
except:
    ERROR = _( 'You need to install gtkspell to use Spell plugin' )

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'SpellCheck for emesene' )
        self.authors = { 'Roger Duran' : 'RogerDuran at gmail dot com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _( 'Spell' )
        self.name = 'Spell'

        self.config = Plugin.Config( 'Spell' )

        self.controller = controller
                
        self.lang = self.config.getValue( self.msn._user+'_lang', '' )
        
        
    def onInputFormatChanged( self, textView ):
        tag = textView.get_buffer().get_tag_table().lookup( 'gtkspell-misspelled' )
        if tag != None:
            tag.set_priority( textView.get_buffer().get_tag_table().get_size() - 1 )
    
    def start( self ):
        '''start the plugin'''
        self.connect( 'new conversation ui', self.newConversationWindow )
        self.connect('close conversation ui', self.closeConversationWindow)
        self.msn.connect( 'input format changed', self.onInputFormatChanged )
            
        self.enabled = True
        
        self.appylAllConv( self.setSpell )
        self.appylAllConv( self.setLang )
        
    def stop( self ):    
        '''stop the plugin'''
        self.disconnect( 'new conversation ui', self.newConversationWindow )
        self.disconnect('close conversation ui', self.closeConversationWindow)
        self.disconnect( 'input format changed', self.onInputFormatChanged )

        self.appylAllConv( self.removeSpell )
        
        self.enabled = False
        
        
    def appylAllConv( self, command ):
        if self.enabled:
            if self.controller.conversationManager != None:
                if len( self.controller.conversationManager.conversations ) > 0:
                    conversations = self.controller.conversationManager.conversations
                    for (key,value) in conversations.iteritems():
                        textView=value[1].ui.input.input
                        try:
                            command( textView )
                        except:
                            pass

        
    def removeSpell( self, textView ):
        gtkspell.get_from_text_view( textView ).detach()
        
    def setSpell( self, textView ):
        gtkspell.Spell( textView )
        self.setLang( textView )
        
    def setLang( self, textView ):
        if self.lang == '':
            self.lang = None
        try:
            gtkspell.get_from_text_view( textView ).set_language( self.lang )
        except:
            print _( 'Something wrong with gtkspell, this language exist'), self.lang,'?'
        
    def action( self ):
        pass
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        if ERROR != '':
            return ( False, ERROR )
            
        return ( True, 'Ok' )

    def newConversationWindow( self, win ):
        textView = win.conversation.ui.getInput().input
        self.setSpell( textView )
        
    def closeConversationWindow( self, win ):
        textView = win.conversation.ui.getInput().input
        try:
            gtkspell.get_from_text_view( textView ).detach()
        except:
            pass

    def configure( self ):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        
        langs =commands.getoutput('aspell dump dicts').split('\n')
        
        l = [ Plugin.Option( 'lang', list, _( 'Default language' ), _( 'Set the default language' ), self.config.getValue( self.msn._user+'_lang', '' ), langs ) ]
        
        response = Plugin.ConfigWindow( _( 'Spell configuration' ), l ).run()
        if response != None:
            if response.has_key( 'lang' ):
                self.config.setValue( self.msn._user+'_lang', str( response[ 'lang' ].value ) )

        self.lang = self.config.getValue( self.msn._user+'_lang', '' )

        self.appylAllConv( self.setLang )
        
        self.config.writeConfig()
