#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#    This plugin uses some code of http://dev.jmoiron.net/projects/browser/xchat/mp3.py?rev=27&format=txt
#

VERSION = '0.3'

import os
import gobject
import gtk
import sys, struct
import socket, pwd


ERROR = ''

#Commands for sockets
CMD_GET_VERSION = 0         #
CMD_PLAYLIST_ADD = 1        #
CMD_PLAY = 2                #
CMD_PAUSE = 3               #
CMD_STOP = 4                #
CMD_IS_PLAYING = 5          #
CMD_IS_PAUSED = 6           #
CMD_GET_PLAYLIST_POS = 7    #
CMD_SET_PLAYLIST_POS = 8    #
CMD_GET_PLAYLIST_LENGTH = 9 #
CMD_PLAYLIST_CLEAR = 10     #
CMD_GET_OUTPUT_TIME = 11    #
CMD_JUMP_TO_TIME = 12       #
CMD_GET_VOLUME = 13         #
CMD_SET_VOLUME = 14         #
CMD_GET_SKIN = 15           #
CMD_SET_SKIN = 16
CMD_GET_PLAYLIST_FILE = 17  #
CMD_GET_PLAYLIST_TITLE = 18 #
CMD_GET_PLAYLIST_TIME = 19  #
CMD_GET_INFO = 20           #
CMD_GET_EQ_DATA = 21 # these two aren't implemented in
CMD_SET_EQ_DATA = 22 # xmms 1.0.1, and that's all I have
CMD_PL_WIN_TOGGLE = 23      #
CMD_EQ_WIN_TOGGLE = 24      #
CMD_SHOW_PREFS_BOX = 25     #
CMD_TOGGLE_AOT = 26         #
CMD_SHOW_ABOUT_BOX = 27     # doesn't seem to work :-(
CMD_EJECT = 28              #
CMD_PLAYLIST_PREV = 29      #
CMD_PLAYLIST_NEXT = 30      #
CMD_PING = 31
CMD_GET_BALANCE = 32        #
CMD_TOGGLE_REPEAT = 33      #
CMD_TOGGLE_SHUFFLE = 34     #
CMD_MAIN_WIN_TOGGLE = 35    #
CMD_PLAYLIST_ADD_URL_STRING = 36
CMD_IS_EQ_WIN = 37          #
CMD_IS_PL_WIN = 38          #
CMD_IS_MAIN_WIN = 39        #

import Plugin

from emesenecommon import *

import gettext

gettext.bindtextdomain('plugin_xmms', LANG_PATH)
gettext.textdomain('plugin_xmms')
_ = gettext.gettext


class ClientPacketHeader:
    def __init__(self):
        self.version,self.cmd,self.length = 0,0,0
    def __repr__(self):
        return "<< %s : version: %s cmd: %s length: %s >>"\
            %(self.__class__.__name__,self.version,self.cmd,self.length)
    def encode(self):
        return struct.pack("hhl",self.version,self.cmd,self.length)

class XmmsSocket:
    def __init__(self,session=0):
        self.status = True
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            self.sock.connect("/tmp/xmms_%s.%d"%(pwd.getpwuid(os.geteuid())[0],session))
        except:
            self.status = False
        
    def send(self,cmd,data=""):
        if type(data) == type(1):
            data = struct.pack("l",data)
        packet = struct.pack("hhl",1,cmd,len(data))+data
        self.sock.send(packet)
    def sendf(self,cmd,fmt,args):
        data = apply(struct.pack,(fmt,)+args)
        self.send(cmd,data)

    def read_header(self, sock):
        head = ClientPacketHeader()
        head.version, head.cmd, head.length = \
            struct.unpack("hhl",sock.recv(8))
        return head

    def get_reply(self):
        self.reply_header = self.read_header(self.sock)
        return self.sock.recv(self.reply_header.length)
    def get_replyf(self,fmt):
        return struct.unpack(fmt,self.get_reply())
    def read_ack(self):
        self.get_reply()
        
    def isXmmsOpen(self):
        return self.status

class XmmsCommands:
    def __init__(self):
        pass
        
    def send_requestf(self, cmd, fmt, data=""):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.send(cmd, data)
            return xmmsSocket.get_replyf(fmt)
        return False
        
    def send_command(self, cmd, data=""):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.send(cmd,data)
            #xmmsSocket.read_ack() 
        return False

    def send_request_args(self, cmd, fmt, *args):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.sendf(cmd, fmt, args)
            return xmmsSocket.get_reply()
        return False
        
    def playlist_pos(self):
        return self.send_requestf(CMD_GET_PLAYLIST_POS,'l')[0]
        
    def playlist_file(self , pos):
        return self.send_request_args(CMD_GET_PLAYLIST_FILE,'l',pos)[:-1]
        
    def playlist_title(self, pos):
        return self.send_request_args(CMD_GET_PLAYLIST_TITLE,'l',pos)[:-1]
        
    def playlist_time(self, pos):
        return self.send_requestf_args(CMD_GET_PLAYLIST_TIME,'l','l',pos)[0]
        
    def socket_info(self):
        """returns (bitrate, freq, nch)"""
        return self.send_requestf(CMD_GET_INFO, 'lll')

    def socket_playlist_length(self):
        return self.send_requestf(CMD_GET_PLAYLIST_LENGTH,'l')[0]

    # in milliseconds
    def socket_output_time(self):
        return self.send_requestf(CMD_GET_OUTPUT_TIME,'l')[0]

    def currentSong(self):
        return self.playlist_title(self.playlist_pos())

    def nextSong( self ):
        print self.send_command(CMD_PLAYLIST_NEXT)
    
    def isPlaying(self):
        if XmmsSocket().isXmmsOpen():
            return self.send_requestf(CMD_IS_PLAYING,'l')[0]
                
        return False


class Xmms:

    def __init__( self ):
        self.currentSong = ''
        self.xmms = XmmsCommands()
        
    def getSongPlain( self ):
        if not self.isPlaying():
            return "not playing"
        return self.xmms.currentSong()
        
    def getCurrentSong( self ):
        if not self.isPlaying():
            return "not playing"
        song = '\\0Music\\01\\0' + self.xmms.currentSong() + '\\0\\0'
        return song
            
    def nextSong( self ):
            return self.xmms.nextSong()
            
    def isPlaying( self ):
        if self.xmms.isPlaying():
            return True
            
        return False
        
    def check( self ):
        if self.currentSong != self.getCurrentSong():
            self.currentSong = self.getCurrentSong()
            return True
                
        return False
    
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn, 1000 )
        self.theme = controller.theme

        self.controller = controller
        self.Slash = controller.Slash
        
        self.description = _( 'Show a personal message with the current song played in Xmms.' )
        self.authors = { 'Roger Duran' : 'RogerDuran at gmail dot com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _( 'Xmms' )
        self.name = 'Xmms'

        self.xmms = Xmms()

        self.config = Plugin.Config( 'Xmms' )
        self.canPlay = int(self.config.getValue( 'canPlay', '1' ))

        self.action()
        
    def start( self ):
        '''start the plugin'''
        
        gobject.timeout_add( self.period, self.action )

        self.addToggleButton()
        
        self.Slash.register('xmms', self.slashCommand, 'Solo muestra la cancion que estamos escuchando a quien le estamos hablando..')
        
        self.enabled = True

    def slashCommand( self, data=None ):
        if data == 'next':
            self.xmms.nextSong()
            return ['New song: '+self.xmms.getSongPlain(), False]
        elif data == 'current':
            return ['Estoy escuchando: '+self.xmms.getSongPlain(), True]
        else:
            return ['Commands : next, current', False]

    def addToggleButton(self):
        self.userPanel = self.controller.mainWindow.userPanel
        self.personalMsg = self.userPanel.tPersonalMessage
        self.pVbox = self.personalMsg.parent        
        
        b = gtk.ToggleButton("\xe2\x99\xab")
        
        b.set_active(self.canPlay)
        
        b.connect("toggled", self.onToggleMusic)
        
        self.pVbox.remove(self.personalMsg)
        self.hbox = gtk.HBox()

        self.mediaEntry = gtk.Entry()
        self.mediaEntry.set_property("editable", False)

        self.personalMsg.hide()
        

        self.hbox.pack_start( self.personalMsg , True , True )
        self.hbox.pack_start( self.mediaEntry , True , True )
        self.hbox.pack_start( b , False , False )

        self.pVbox.pack_start( self.hbox , True , True )
        
        self.mediaEntry.show()
        b.show()
        
        self.hbox.show()
        
        
    def delToggleButton(self):
        self.hbox.remove(self.personalMsg)
        self.pVbox.remove(self.hbox)
        self.pVbox.pack_start( self.personalMsg , True , True )
        self.pVbox.show_all()
        
    def onToggleMusic( self, toggleButton, data='' ):
      if toggleButton.get_active():
          self.canPlay = True
          self.personalMsg.hide()
          self.mediaEntry.show()
      else:
          self.canPlay = False
          self.mediaEntry.hide()
          self.personalMsg.show()

      self.config.setValue( 'canPlay', str( int( self.canPlay ) ) )


    def stop( self ):    
        '''stop the plugin'''
        
        self.msn.changeCurrentMedia( "" )
        self.delToggleButton()
        self.Slash.unregister('xmms')
        self.enabled = False
        
    def action( self ):
        if self.enabled:
            if self.canPlay and self.xmms.isPlaying():
                if self.xmms.check():
                    self.msn.changeCurrentMedia( self.xmms.currentSong )
                    self.mediaEntry.set_text( "\xe2\x99\xab " +self.xmms.getSongPlain() + " \xe2\x99\xab" )
            elif self.xmms.currentSong != '':
                self.xmms.currentSong = ''
                self.mediaEntry.set_text( "\xe2\x99\xab " + _( 'Player Stopped' ) + " \xe2\x99\xab" )
                self.msn.changeCurrentMedia( '' )

            return True
        else:
            return False
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True
        
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if os.name != 'posix':
            return ( False, _( 'This plugin only works in posix systems' ) )

        if ERROR:
            return ( False, ERROR )
        
        return ( True, 'Ok' )
